@@ -4,7 +4,7 @@
  *
  *	Created by:	C. Anthony Della Fera
  *
- *	$Id$
+ *	$Id: 2f51cc05b9f2db41c73282c5f50cb25bf9c2ce89 $
  *
  *	Copyright (c) 1987, 1993 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -21,7 +21,7 @@
 
 #ifndef lint
 static const char rcsid_zshutdown_notify_c[] =
-    "$Id$";
+    "$Id: 2f51cc05b9f2db41c73282c5f50cb25bf9c2ce89 $";
 #endif
 
 #define N_KIND		UNSAFE
@@ -92,7 +92,7 @@ main(int argc,
     }
     retval = krb_get_svc_in_tkt(SVC_NAME, hn2, rlm,
 				SERVER_SERVICE, SERVER_INSTANCE, 1,
-				KEYFILE);
+				(char *)KEYFILE);
     if (retval) {
 	fprintf(stderr, "%s: can't get tickets: %s\n",
 		argv[0], krb_get_err_text(retval));
