@@ -3,7 +3,7 @@
  *
  *      Created by:     David C. Jedlinsky
  *
- *      $Id$
+ *      $Id: 7ae5954a6f95aadbaf00c9f9373bfe16f0507895 $
  *
  *      Copyright (c) 1987,1988 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -21,7 +21,7 @@
 #include "zserver.h"
 
 #if !defined(lint) && !defined(SABER)
-static const char rcsid_zstat_c[] = "$Id$";
+static const char rcsid_zstat_c[] = "$Id: 7ae5954a6f95aadbaf00c9f9373bfe16f0507895 $";
 #endif
 
 const char *hm_head[] = {
@@ -55,7 +55,7 @@ void do_stat(char *);
 int srv_stat(char *);
 int hm_stat(char *, char *);
 
-RETSIGTYPE
+static RETSIGTYPE
 timeout(int ignored)
 {
 	outoftime = 1;
@@ -69,8 +69,6 @@ main(int argc,
 	char hostname[NS_MAXDNAME];
 	int optchar;
 	struct servent *sp;
-	extern char *optarg;
-	extern int optind;
 
 	if ((ret = ZInitialize()) != ZERR_NONE) {
 		com_err("zstat", ret, "initializing");
@@ -145,7 +143,7 @@ hm_stat(char *host,
 	Code_t code;
 
 	char *line[20],*mp;
-	int i,nf;
+	unsigned int i,nf;
 	struct hostent *hp;
 	time_t runtime;
 	struct tm *tim;
