@@ -3,7 +3,7 @@
  *
  *	Created by:	Robert French
  *
- *	$Id$
+ *	$Id: aae3101dcb16a10f1c98cf9b40a56512b2d837b4 $
  *
  *	Copyright (c) 1987,1988 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -17,7 +17,7 @@
 #include <pwd.h>
 
 #ifndef lint
-static const char rcsid_zwrite_c[] = "$Id$";
+static const char rcsid_zwrite_c[] = "$Id: aae3101dcb16a10f1c98cf9b40a56512b2d837b4 $";
 #endif /* lint */
 
 #define DEFAULT_CLASS "MESSAGE"
@@ -29,8 +29,9 @@ static const char rcsid_zwrite_c[] = "$Id$";
 #define MAXRECIPS 100
 
 int nrecips, msgarg, verbose, quiet, nodot, cc;
-char *whoami, *inst, *class, *opcode, *realm, *recips[MAXRECIPS];
+char *whoami, *inst, *class, *opcode, *realm, *recips[MAXRECIPS], *sender = 0;
 Z_AuthProc auth;
+ZNotice_Kind_t kind = ACKED;
 void un_tabify(char **, int *);
 
 char *fix_filsrv_inst(char *);
@@ -189,6 +190,17 @@ main(int argc, char *argv[])
 	    arg++;
 	    charset = argv[arg];
 	    break;
+	case 'S':
+	    if (arg == argc-1)
+		usage(whoami);
+	    arg++;
+	    sender = argv[arg];
+	    auth = ZNOAUTH;
+	    break;
+	case 'U':
+	    kind = UNACKED;
+	    nocheck = 1;		/* implied -n (no ping) */
+	    break;
 	default:
 	    usage(whoami);
 	}
@@ -224,32 +236,22 @@ main(int argc, char *argv[])
 	    }
 	    signature = sigbfr;
 	}
-    }	
+    }
 
-    notice.z_kind = ACKED;
+    memset(&notice, 0, sizeof(ZNotice_t));
+    notice.z_kind = kind;
     notice.z_port = 0;
     notice.z_class = class;
     notice.z_class_inst = inst;
     notice.z_opcode = "PING";
-    notice.z_sender = 0;
+    notice.z_sender = sender;
     notice.z_message_len = 0;
     notice.z_recipient = "";
     notice.z_charset = ZGetCharset(charset);
     if (format)
 	    notice.z_default_format = format;
-    else if (filsys == 1)
-	    notice.z_default_format = "@b(Filesystem Info Message for $instance)\nFrom: @b($sender) @ $time $date\n$message";
-    else if (auth == ZAUTH) {
-	if (signature)
-	    notice.z_default_format = "Class $class Instance $instance\nTo @b($recipient) @ $time $date\nFrom @b($1) <$sender>\n\n$2";
-	else
-	    notice.z_default_format = "Class $class Instance $instance\nTo @b($recipient) @ $time $date\n$message";
-    } else {
-	if (signature)
-	    notice.z_default_format = "@b(UNAUTHENTIC) Class $class Instance $instance @ $time $date\nFrom @b($1) <$sender>\n\n$2";
-	else
-	    notice.z_default_format = "@b(UNAUTHENTIC) Class $class Instance $instance @ $time $date\n$message";
-    }
+    else
+	    notice.z_default_format = "http://zephyr.1ts.org/wiki/df";
     if (!nocheck && nrecips)
 	send_off(&notice, 0);
 	
@@ -355,7 +357,7 @@ void
 send_off(ZNotice_t *notice, int real)
 {
     int i, success, retval;
-    char bfr[BUFSIZ], realm_recip[BUFSIZ], dest[3 * BUFSIZ];
+    char realm_recip[BUFSIZ], dest[3 * BUFSIZ];
     ZNotice_t retnotice;
 
     success = 0;
@@ -381,10 +383,11 @@ send_off(ZNotice_t *notice, int real)
 		   class, inst, 
 		   nrecips?notice->z_recipient:"everyone");
 	if ((retval = ZSendNotice(notice, auth)) != ZERR_NONE) {
-	    (void) sprintf(bfr, "while sending notice to %s", dest);
-	    com_err(whoami, retval, bfr);
+	    com_err(whoami, retval, "while sending notice to %s", dest);
 	    break;
 	}
+	if (kind == UNACKED)
+	    continue;
 	if (real && !quiet) {
 	    if (verbose)
 		printf("Queued... ");
@@ -395,9 +398,8 @@ send_off(ZNotice_t *notice, int real)
 	if ((retval = ZIfNotice(&retnotice, (struct sockaddr_in *) 0,
 				ZCompareUIDPred, 
 				(char *)&notice->z_uid)) != ZERR_NONE) {
-	    (void) sprintf(bfr, "while waiting for acknowledgement for %s", 
+	    com_err(whoami, retval, "while waiting for acknowledgement for %s",
 		    dest);
-	    com_err(whoami, retval, bfr);
 	    continue;
 	}
 	if (retnotice.z_kind == SERVNAK) {
