@@ -3,7 +3,7 @@
  *
  *	Created by:	Robert French
  *
- *	$Id$
+ *	$Id: 7f8947ff4246ccbf217be749e7ec73861a60ca90 $
  *
  *	Copyright (c) 1987 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -11,11 +11,15 @@
  */
 
 #ifndef lint
-static const char rcsid_ZGetSubscriptions_c[] = "$Id$";
+static const char rcsid_ZGetSubscriptions_c[] = "$Id: 7f8947ff4246ccbf217be749e7ec73861a60ca90 $";
 #endif
 
 #include <internal.h>
 
+#ifndef MIN
+#define MIN(x, y)               ((x) < (y) ? (x) : (y))
+#endif
+
 Code_t
 ZGetSubscriptions(ZSubscription_t *subscription,
 		  int *numsubs)
