@@ -3,7 +3,7 @@
  *
  *	Created by:	Robert French
  *
- *	$Id$
+ *	$Id: 69c1ecdbafc34a54ef331519b588e0b710c4f5c5 $
  *
  *	Copyright (c) 1987, 1990 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static const char rcsid_ZReadAscii_c[] = "$Id$";
+static const char rcsid_ZReadAscii_c[] = "$Id: 69c1ecdbafc34a54ef331519b588e0b710c4f5c5 $";
 #endif /* lint */
 
 #include <internal.h>
@@ -28,7 +28,7 @@ ZReadAscii(char *ptr,
     int i;
     unsigned int hexbyte;
     register int c1, c2;
-    register unsigned int temp;
+    register int temp;
 
     for (i=0;i<num;i++) {
 	if (len >= 1 && *ptr == ' ') {
