@@ -3,7 +3,7 @@
  *
  *	Created by:	Robert French
  *
- *	$Id$
+ *	$Id: e3b14a4931628a7454f2b0b9c1cf58525fe2980f $
  *
  *	Copyright (c) 1987,1988 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static const char rcsid_ZReceiveNotice_c[] = "$Id$";
+static const char rcsid_ZReceiveNotice_c[] = "$Id: e3b14a4931628a7454f2b0b9c1cf58525fe2980f $";
 #endif
 
 #include <internal.h>
@@ -29,6 +29,8 @@ ZReceiveNotice(ZNotice_t *notice,
 	return (retval);
 
     nextq = Z_GetFirstComplete();
+    if (!nextq)
+      return (ENOMEM);
 
     len = nextq->packet_len;
     
