@@ -3,7 +3,7 @@
  *
  *	Created by:	Robert French
  *
- *	$Id$
+ *	$Id: 4fc246d3ae66beada0ff1c2bdf7a61443739282e $
  *
  *	Copyright (c) 1987 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -11,7 +11,7 @@
  */
 
 #ifndef lint
-static const char rcsid_ZSendNotice_c[] = "$Id$";
+static const char rcsid_ZSendNotice_c[] = "$Id: 4fc246d3ae66beada0ff1c2bdf7a61443739282e $";
 #endif
 
 #include <internal.h>
@@ -38,8 +38,11 @@ ZSrvSendNotice(ZNotice_t *notice,
 				cert_routine)) != ZERR_NONE)
 	return (retval);
 
-    if ((retval = ZParseNotice(buffer, len, &newnotice)) != ZERR_NONE)
+    if ((retval = ZParseNotice(buffer, len, &newnotice)) != ZERR_NONE) {
+	free(buffer);
 	return (retval);
+    }
+
     
     retval = Z_SendFragmentedNotice(&newnotice, len, cert_routine,
 				    send_routine);
