@@ -4,7 +4,7 @@
  *
  *	Created by:	Robert French
  *
- *	$Id$
+ *	$Id: 535f3f996b103138c9ef72a0ec1ab082cde0aa16 $
  *
  *	Copyright (c) 1987,1988 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -14,14 +14,11 @@
 #include <internal.h>
 
 #ifndef lint
-static const char rcsid_ZSubscriptions_c[] = "$Id$";
+static const char rcsid_ZSubscriptions_c[] = "$Id: 535f3f996b103138c9ef72a0ec1ab082cde0aa16 $";
 #endif
 
-static Code_t Z_Subscriptions(register ZSubscription_t *sublist,
-			      int nitems, unsigned int port,
-			      char *opcode, int authit);
-static Code_t subscr_sendoff(ZNotice_t *notice, char **lyst, int num,
-			     int authit);
+static Code_t Z_SendAndWaitForServer(ZNotice_t *notice, char *buf, int len,
+				     int waitforack);
 
 #ifdef CMU_ZCTL_PUNT
 Code_t
@@ -29,7 +26,8 @@ ZPunt(ZSubscription_t *sublist,
       int nitems,
       unsigned int port)
 {
-    return (Z_Subscriptions(sublist, nitems, port, "SUPPRESS", 1));
+    return (ZSubscriptions(sublist, nitems, port, "SUPPRESS",
+			   Z_SendAndWaitForServer));
 }
 #endif
 
@@ -38,7 +36,8 @@ ZSubscribeTo(ZSubscription_t *sublist,
 	     int nitems,
 	     unsigned int port)
 {
-    return (Z_Subscriptions(sublist, nitems, port, CLIENT_SUBSCRIBE, 1));
+    return (ZSubscriptions(sublist, nitems, port, CLIENT_SUBSCRIBE,
+			   Z_SendAndWaitForServer));
 }
 
 Code_t
@@ -46,8 +45,8 @@ ZSubscribeToSansDefaults(ZSubscription_t *sublist,
 			 int nitems,
 			 unsigned int port)
 {
-    return (Z_Subscriptions(sublist, nitems, port, CLIENT_SUBSCRIBE_NODEFS,
-			    1));
+    return (ZSubscriptions(sublist, nitems, port, CLIENT_SUBSCRIBE_NODEFS,
+			   Z_SendAndWaitForServer));
 }
 
 Code_t
@@ -55,14 +54,15 @@ ZUnsubscribeTo(ZSubscription_t *sublist,
 	       int nitems,
 	       unsigned int port)
 {
-    return (Z_Subscriptions(sublist, nitems, port, CLIENT_UNSUBSCRIBE, 1));
+    return (ZSubscriptions(sublist, nitems, port, CLIENT_UNSUBSCRIBE,
+			   Z_SendAndWaitForServer));
 }
 
 Code_t
 ZCancelSubscriptions(unsigned int port)
 {
-    return (Z_Subscriptions((ZSubscription_t *)0, 0, port,
-			    CLIENT_CANCELSUB, 0));
+    return (ZSubscriptions((ZSubscription_t *)0, 0, port, CLIENT_CANCELSUB,
+			   Z_SendAndWaitForServer));
 }
 
 /*
@@ -71,12 +71,12 @@ ZCancelSubscriptions(unsigned int port)
  * mis-interpret them.
  */
 
-static Code_t
-Z_Subscriptions(register ZSubscription_t *sublist,
-		int nitems,
-		unsigned int port,
-		char *opcode,
-		int authit)
+Code_t
+ZSubscriptions(register ZSubscription_t *sublist,
+	       int nitems,
+	       unsigned int port,
+	       char *opcode,
+	       Code_t (*send_routine)(ZNotice_t *, char *, int, int))
 {
     register int i, j;
     int retval;
@@ -85,9 +85,13 @@ Z_Subscriptions(register ZSubscription_t *sublist,
     char **list;
     char *recip;
     int hdrlen;
-    int size_avail = Z_MAXPKTLEN-Z_FRAGFUDGE; /* space avail for data,
-						 adjusted below */
+    /* Space available for data, adjusted below. Take off Z_FRAGFUDGE twice.
+       The first is to account for Z_SendFragmentedNotice's space. The second
+       to account for hdrlen not being constant. Zcode escapes bytes 0x00 and
+       0xFF, so some bytes are encoded as two bytes. */
+    int size_avail = Z_MAXPKTLEN-Z_FRAGFUDGE-Z_FRAGFUDGE;
     int size, start, numok;
+    Z_AuthProc cert_routine;
 
     /* nitems = 0 means cancel all subscriptions; still need to allocate a */
     /* array for one item so we can cancel, however. */
@@ -109,14 +113,19 @@ Z_Subscriptions(register ZSubscription_t *sublist,
 
     /* format the header to figure out how long it is */
     retval = Z_FormatHeader(&notice, header, sizeof(header), &hdrlen, ZAUTH);
-    if (retval != ZERR_NONE && !authit)
-	retval = Z_FormatHeader(&notice, header, sizeof(header),
-				&hdrlen, ZNOAUTH);
     if (retval != ZERR_NONE) {
 	free((char *)list);
 	return(retval);
     }
 
+    /* if a subscription request, we save the key */
+    if (strcmp(opcode, CLIENT_SUBSCRIBE) == 0 ||
+	strcmp(opcode, CLIENT_SUBSCRIBE_NODEFS) == 0) {
+	cert_routine = ZSUBAUTH;
+    } else {
+	cert_routine = ZAUTH;
+    }
+
     /* compute amount of room left */
     size_avail -= hdrlen;
     size = size_avail;
@@ -138,7 +147,7 @@ Z_Subscriptions(register ZSubscription_t *sublist,
     numok = 0;
     if (!nitems) {
 	/* there aren't really any, but we need to xmit anyway */
-	retval = subscr_sendoff(&notice, list, 0, authit);
+	retval = ZSrvSendList(&notice, list, 0, cert_routine, send_routine);
 	free((char *)list);
 	return(retval);
     }
@@ -162,7 +171,8 @@ Z_Subscriptions(register ZSubscription_t *sublist,
 	    free((char *)list);
 	    return(ZERR_FIELDLEN);
 	}
-	retval = subscr_sendoff(&notice, &list[start*3], numok, authit);
+	retval = ZSrvSendList(&notice, &list[start*3], numok * 3,
+			      cert_routine, send_routine);
 	if (retval) {
 	    free((char *)list);
 	    return(retval);
@@ -170,26 +180,28 @@ Z_Subscriptions(register ZSubscription_t *sublist,
 	start = -1;
     }
     if (numok)
-	retval = subscr_sendoff(&notice, &list[start*3], numok, authit);
+	retval = ZSrvSendList(&notice, &list[start*3], numok * 3,
+			      cert_routine, send_routine);
     free((char *)list);
     return(retval);
 }
 
 static Code_t
-subscr_sendoff(ZNotice_t *notice,
-	       char **lyst,
-	       int num,
-	       int authit)
+Z_SendAndWaitForServer(ZNotice_t *notice,
+		       char *buf, int len,
+		       int waitforack)
 {
-    register Code_t retval;
+    Code_t retval;
     ZNotice_t retnotice;
 
-    retval = ZSendList(notice, lyst, num*3, ZAUTH);
-    if (retval != ZERR_NONE && !authit)
-	retval = ZSendList(notice, lyst, num*3, ZNOAUTH);
-	
+    retval = ZSendPacket(buf, len, waitforack);
     if (retval != ZERR_NONE)
 	return (retval);
+    /* Z_ReadWait drops non-initial SERVACKs and SERVNAKs on the floor. We
+       should never see a non-initial one here, but be defensive about bugs in the
+       sharding code above. */
+    if (!ZCompareUID(&notice->z_multiuid, &notice->z_uid))
+	return (retval);
     if ((retval = ZIfNotice(&retnotice, (struct sockaddr_in *)0, 
 				ZCompareUIDPred, (char *)&notice->z_uid)) !=
 	ZERR_NONE)
@@ -205,3 +217,42 @@ subscr_sendoff(ZNotice_t *notice,
     ZFreeNotice(&retnotice);
     return (ZERR_NONE);
 }
+
+Code_t
+ZFlushUserSubscriptions(char *recip)
+{
+    register Code_t retval;
+    ZNotice_t notice, retnotice;
+
+    (void)memset((char *)&notice, 0, sizeof(notice));
+    notice.z_kind = ACKED;
+    notice.z_class = ZEPHYR_CTL_CLASS;
+    notice.z_class_inst = ZEPHYR_CTL_CLIENT;
+    notice.z_opcode = CLIENT_FLUSHSUBS;
+    notice.z_recipient = "";
+    notice.z_default_format = "";
+    if (recip) {
+	notice.z_message = recip;
+	notice.z_message_len = strlen(recip) + 1;
+    } else {
+	notice.z_message_len = 0;
+    }
+
+    if ((retval = ZSendNotice(&notice, ZAUTH)) != ZERR_NONE)
+	return (retval);
+
+    if ((retval = ZIfNotice(&retnotice, (struct sockaddr_in *)0,
+			    ZCompareUIDPred, (char *)&notice.z_uid)) !=
+	ZERR_NONE)
+	return (retval);
+    if (retnotice.z_kind == SERVNAK) {
+	ZFreeNotice(&retnotice);
+	return (ZERR_SERVNAK);
+    }
+    if (retnotice.z_kind != SERVACK) {
+	ZFreeNotice(&retnotice);
+	return (ZERR_INTERNAL);
+    }
+    ZFreeNotice(&retnotice);
+    return (ZERR_NONE);
+}
