@@ -3,7 +3,7 @@
  *
  *	Created by:	Robert French
  *
- *	$Id$
+ *	$Id: ecee122f62a4c55ec5f3dc829eb396ae00078670 $
  *
  *	Copyright (c) 1987,1988,1991 by the Massachusetts Institute of
  *	Technology.
@@ -17,7 +17,7 @@
 
 #ifndef lint
 static const char rcsid_Zinternal_c[] =
-  "$Id$";
+  "$Id: ecee122f62a4c55ec5f3dc829eb396ae00078670 $";
 static const char copyright[] =
   "Copyright (c) 1987,1988,1991 by the Massachusetts Institute of Technology.";
 #endif
@@ -44,8 +44,10 @@ int Z_discarded_packets = 0;
 #ifdef HAVE_KRB5
 /* This context is used throughout */
 krb5_context Z_krb5_ctx;
+/* A queue of all the currently active session keys. */
+struct _Z_SessionKey *Z_keys_head, *Z_keys_tail;
 
-static struct cksum_map_s {
+static const struct cksum_map_s {
   krb5_enctype e;
   krb5_cksumtype c;
 } cksum_map[] = {
@@ -78,6 +80,8 @@ static struct cksum_map_s {
   /* per draft-brezak-win2k-krb-rc4-hmac-04.txt */
   { 23 /* rc4-hmac */,                -138 /* hmac-md5 */ },
   { 24 /* rc4-hmac-exp */,            -138 /* hmac-md5 */ },
+  { 25 /* camellia128-cts-cmac */,    17 /* cmac-camellia128 */ },
+  { 26 /* camellia256-cts-cmac */,    18 /* cmac-camellia256 */ },
 };
 #define N_CKSUM_MAP (sizeof(cksum_map) / sizeof(struct cksum_map_s))
 
@@ -85,7 +89,7 @@ Code_t
 Z_krb5_lookup_cksumtype(krb5_enctype e,
 			krb5_cksumtype *c)
 {
-  int i;
+  unsigned int i;
 
   for (i = 0; i < N_CKSUM_MAP; i++) {
     if (cksum_map[i].e == e) {
@@ -109,7 +113,8 @@ void *__Z_debug_print_closure;
 static int Z_AddField(char **ptr, char *field, char *end);
 static int find_or_insert_uid(ZUnique_Id_t *uid, ZNotice_Kind_t kind);
 static Code_t Z_ZcodeFormatRawHeader(ZNotice_t *, char *, int, int *, char **,
-				     int *, char **, char **, int cksumtype);
+				     int *, char **, char **, int cksumstyle,
+				     int addrstyle);
 
 /* Find or insert uid in the old uids buffer.  The buffer is a sorted
  * circular queue.  We make the assumption that most packets arrive in
@@ -131,8 +136,8 @@ find_or_insert_uid(ZUnique_Id_t *uid,
 
     time_t now;
     struct _filter *new;
-    long i, new_size;
-    int result;
+    long i, j, new_size;
+    int result, found = 0;
 
     /* Initialize the uid buffer if it hasn't been done already. */
     if (!buffer) {
@@ -142,9 +147,9 @@ find_or_insert_uid(ZUnique_Id_t *uid,
 	    return 0;
     }
 
-    /* Age the uid buffer, discarding any uids older than the clock skew. */
+    /* Age the uid buffer, discarding any uids older than the time limit. */
     time(&now);
-    while (num && (now - buffer[start % size].t) > CLOCK_SKEW)
+    while (num && (now - buffer[start % size].t) > Z_FILTERTIMELIMIT)
 	start++, num--;
     start %= size;
 
@@ -165,18 +170,25 @@ find_or_insert_uid(ZUnique_Id_t *uid,
     /* Search for this uid in the buffer, starting from the end. */
     for (i = start + num - 1; i >= start; i--) {
 	result = memcmp(uid, &buffer[i % size].uid, sizeof(*uid));
-	if (result == 0 && buffer[i % size].kind == kind)
-	    return 1;
+	if (result == 0 && buffer[i % size].kind == kind) {
+	    /* Remove it from the buffer. We'll re-add it at the end. */
+	    for (j = i; j < start + num - 1; j++) {
+		buffer[j % size] = buffer[(j + 1) % size];
+	    }
+	    num--;
+	    found = 1;
+	    break;
+        }
     }
 
-    /* We didn't find it; stick it on the end */
+    /* Whether or not we found it, stick it at the end. */
     i = start + num;
     buffer[i % size].uid = *uid;
     buffer[i % size].kind = kind;
     buffer[i % size].t = now;
     num++;
 
-    return 0;
+    return found;
 }
 
 
@@ -186,12 +198,12 @@ int
 Z_PacketWaiting(void)
 {
     struct timeval tv;
-    fd_set read;
+    fd_set readfds;
 
     tv.tv_sec = tv.tv_usec = 0;
-    FD_ZERO(&read);
-    FD_SET(ZGetFD(), &read);
-    return (select(ZGetFD() + 1, &read, NULL, NULL, &tv));
+    FD_ZERO(&readfds);
+    FD_SET(ZGetFD(), &readfds);
+    return (select(ZGetFD() + 1, &readfds, NULL, NULL, &tv));
 }
 
 
@@ -252,7 +264,8 @@ Z_SearchQueue(ZUnique_Id_t *uid,
 	if (ZCompareUID(uid, &qptr->uid) && qptr->kind == kind)
 	    return (qptr);
 	next = qptr->next;
-	if (qptr->timep && (qptr->timep+Z_NOTICETIMELIMIT < tv.tv_sec))
+	if (qptr->timep &&
+            (qptr->timep+Z_NOTICETIMELIMIT < (unsigned long)tv.tv_sec))
 	    Z_RemQueue(qptr);
 	qptr = next;
     }
@@ -281,6 +294,7 @@ Z_ReadWait(void)
     Code_t retval;
     fd_set fds;
     struct timeval tv;
+    ZUnique_Id_t *multiuid;
 
     if (ZGetFD() < 0)
 	return (ZERR_NOPORT);
@@ -310,7 +324,7 @@ Z_ReadWait(void)
     zvlen = sizeof(ZVERSIONHDR) - 1;
     if (packet_len < zvlen || memcmp(packet, ZVERSIONHDR, zvlen) != 0) {
 	Z_discarded_packets++;
-	return (ZERR_NONE);
+	return (ZERR_BADPKT);
     }
 
     /* Parse the notice */
@@ -369,16 +383,18 @@ Z_ReadWait(void)
     if (partof > Z_MAXNOTICESIZE)
 	return (ZERR_NONE);
 
-    /*
-     * If we aren't a server and we can find a notice in the queue
-     * with the same multiuid field, insert the current fragment as
-     * appropriate.
+    /* The packet has garbage at the end. This likely came from a cross-realm
+     * zephyrd without f276622ace757977fec43633e43577350e0cf6fe, which means
+     * it's retransmitting blindly anyway. Drop it on the floor.
      */
+    if (notice.z_message_len > partof - part)
+	return (ZERR_NONE);
+
+    /* Pick the appropriate key to reassemble with. */
     switch (notice.z_kind) {
     case SERVACK:
     case SERVNAK:
-	/* The SERVACK and SERVNAK replies shouldn't be reassembled
-	   (they have no parts).  Instead, we should hold on to the reply
+	/* For SERVACK and SERNACK replies, hold on to the reply
 	   ONLY if it's the first part of a fragmented message, i.e.
 	   multi_uid == uid.  This allows programs to wait for the uid
 	   of the first packet, and get a response when that notice
@@ -390,26 +406,33 @@ Z_ReadWait(void)
 	    !ZCompareUID(&notice.z_multiuid, &notice.z_uid))
 	    /* they're not the same... throw away this packet. */
 	    return(ZERR_NONE);
-	/* fall thru & process it */
+	/* fall thru to general ACK case. */
+    case HMACK:
+	/* The HMACK, SERVACK, and SERVNAK replies shouldn't be
+	   reassembled (they have no parts). */
+	multiuid = &notice.z_uid;
+	break;
     default:
-	/* for HMACK types, we assume no packet loss (local loopback
-	   connections).  The other types can be fragmented and MUST
-	   run through this code. */
-	if (!__Zephyr_server && (qptr = Z_SearchQueue(&notice.z_multiuid,
-						      notice.z_kind))) {
-	    /*
-	     * If this is the first fragment, and we haven't already
-	     * gotten a first fragment, grab the header from it.
-	     */
-	    if (part == 0 && !qptr->header) {
-		qptr->header_len = packet_len-notice.z_message_len;
-		qptr->header = (char *) malloc((unsigned) qptr->header_len);
-		if (!qptr->header)
-		    return (ENOMEM);
-		(void) memcpy(qptr->header, packet, qptr->header_len);
-	    }
-	    return (Z_AddNoticeToEntry(qptr, &notice, part));
+	multiuid = &notice.z_multiuid;
+    }
+    /*
+     * If we aren't a server and we can find a notice in the queue
+     * with the same multiuid field, insert the current fragment as
+     * appropriate.
+     */
+    if (!__Zephyr_server && (qptr = Z_SearchQueue(multiuid, notice.z_kind))) {
+	/*
+	 * If this is the first fragment, and we haven't already
+	 * gotten a first fragment, grab the header from it.
+	 */
+	if (part == 0 && !qptr->header) {
+	    qptr->header_len = packet_len-notice.z_message_len;
+	    qptr->header = (char *) malloc((unsigned) qptr->header_len);
+	    if (!qptr->header)
+		return (ENOMEM);
+	    (void) memcpy(qptr->header, packet, qptr->header_len);
 	}
+	return (Z_AddNoticeToEntry(qptr, &notice, part));
     }
 
     /*
@@ -441,7 +464,7 @@ Z_ReadWait(void)
 
     /* Copy the from field, multiuid, kind, and checked authentication. */
     qptr->from = from;
-    qptr->uid = notice.z_multiuid;
+    qptr->uid = *multiuid;
     qptr->kind = notice.z_kind;
     qptr->auth = notice.z_checked_auth;
 
@@ -564,54 +587,28 @@ Z_AddNoticeToEntry(struct _Z_InputQ *qptr,
 	    qptr->holelist = hole->next;
 	free((char *)hole);
 	/*
-	 * Now create a new hole that is the original hole without the
-	 * current fragment.
+	 * Now create new hole(s) that are the original hole without
+	 * the current fragment.
 	 */
 	if (part > oldfirst) {
-	    /* Search for the end of the hole list */
-	    hole = qptr->holelist;
-	    lasthole = (struct _Z_Hole *) 0;
-	    while (hole) {
-		lasthole = hole;
-		hole = hole->next;
-	    }
-	    if (lasthole) {
-		lasthole->next = (struct _Z_Hole *)malloc(sizeof(struct _Z_Hole));
-		if (lasthole->next == NULL)
-		    return ENOMEM;
-		hole = lasthole->next;
-	    } else {
-		qptr->holelist = (struct _Z_Hole *)malloc(sizeof(struct _Z_Hole));
-		if (qptr->holelist == NULL)
-		    return ENOMEM;
-		hole = qptr->holelist;
-	    }
-	    hole->next = NULL;
+	    hole = (struct _Z_Hole *)malloc(sizeof(struct _Z_Hole));
+	    if (hole == NULL)
+		return ENOMEM;
 	    hole->first = oldfirst;
 	    hole->last = part-1;
+	    /* Prepend to the list; holelist is unordered. */
+	    hole->next = qptr->holelist;
+	    qptr->holelist = hole;
 	}
 	if (last < oldlast) {
-	    /* Search for the end of the hole list */
-	    hole = qptr->holelist;
-	    lasthole = (struct _Z_Hole *) 0;
-	    while (hole) {
-		lasthole = hole;
-		hole = hole->next;
-	    }
-	    if (lasthole) {
-		lasthole->next = (struct _Z_Hole *)malloc(sizeof(struct _Z_Hole));
-		if (lasthole->next == NULL)
-		    return ENOMEM;
-		hole = lasthole->next;
-	    } else {
-		qptr->holelist = (struct _Z_Hole *)malloc(sizeof(struct _Z_Hole));
-		if (qptr->holelist == NULL)
-		    return ENOMEM;
-		hole = qptr->holelist;
-	    }
-	    hole->next = (struct _Z_Hole *) 0;
+	    hole = (struct _Z_Hole *)malloc(sizeof(struct _Z_Hole));
+	    if (hole == NULL)
+		return ENOMEM;
 	    hole->first = last+1;
 	    hole->last = oldlast;
+	    /* Prepend to the list; holelist is unordered. */
+	    hole->next = qptr->holelist;
+	    qptr->holelist = hole;
 	}
     }
 
@@ -740,6 +737,24 @@ Z_NewFormatHeader(ZNotice_t *notice,
     return Z_NewFormatAuthHeader(notice, buffer, buffer_len, len, cert_routine);
 }
 
+Code_t
+Z_FormatAuthHeaderWithASCIIAddress(ZNotice_t *notice,
+				   char *buffer,
+				   int buffer_len,
+				   int *len)
+{
+    notice->z_auth = 0;
+    notice->z_authent_len = 0;
+    notice->z_ascii_authent = "";
+    notice->z_checksum = 0;
+    if (!(notice->z_sender_sockaddr.sa.sa_family == AF_INET ||
+	  notice->z_sender_sockaddr.sa.sa_family == AF_INET6))
+	notice->z_sender_sockaddr.sa.sa_family = AF_INET; /* \/\/hatever *//*XXX*/
+
+    return Z_ZcodeFormatRawHeader(notice, buffer, buffer_len, len,
+				  NULL, NULL, NULL, NULL, 0, 1);
+}
+
 Code_t
 Z_FormatAuthHeader(ZNotice_t *notice,
 		   char *buffer,
@@ -789,7 +804,7 @@ Z_NewFormatRawHeader(ZNotice_t *notice,
 		     char **cend)
 {
    return(Z_ZcodeFormatRawHeader(notice, buffer, buffer_len, hdr_len,
-				 cksum_start, cksum_len, cstart, cend, 0));
+				 cksum_start, cksum_len, cstart, cend, 0, 0));
 }
 
 Code_t
@@ -803,7 +818,7 @@ Z_AsciiFormatRawHeader(ZNotice_t *notice,
 		       char **cend)
 {
    return(Z_ZcodeFormatRawHeader(notice, buffer, buffer_len, hdr_len,
-				 cksum_start, cksum_len, cstart, cend, 1));
+				 cksum_start, cksum_len, cstart, cend, 1, 0));
 }
 
 static Code_t
@@ -815,7 +830,8 @@ Z_ZcodeFormatRawHeader(ZNotice_t *notice,
 		       int *cksum_len,
 		       char **cstart,
 		       char **cend,
-		       int cksumstyle)
+		       int cksumstyle,
+		       int addrstyle)
 {
     static char version_nogalaxy[BUFSIZ]; /* default init should be all \0 */
     char newrecip[BUFSIZ];
@@ -945,8 +961,13 @@ Z_ZcodeFormatRawHeader(ZNotice_t *notice,
 	    addraddr = (unsigned char *)&notice->z_sender_sockaddr.ip6.sin6_addr;
 	}
 
-	if (ZMakeZcode(ptr, end-ptr, addraddr, addrlen) == ZERR_FIELDLEN)
-	    return ZERR_HEADERLEN;
+	if (notice->z_sender_sockaddr.sa.sa_family == AF_INET && addrstyle) {
+	    if (ZMakeAscii(ptr, end-ptr, addraddr, addrlen) == ZERR_FIELDLEN)
+		return ZERR_HEADERLEN;
+	} else {
+	    if (ZMakeZcode(ptr, end-ptr, addraddr, addrlen) == ZERR_FIELDLEN)
+		return ZERR_HEADERLEN;
+	}
 	ptr += strlen(ptr) + 1;
     }
 
@@ -976,132 +997,13 @@ Z_FormatRawHeader(ZNotice_t *notice,
 		  char **cstart,
 		  char **cend)
 {
-    char newrecip[BUFSIZ];
-    char *ptr, *end;
-    int i;
-    int addrlen = 0;
-    unsigned char *addraddr = NULL;
 
     if (!(notice->z_sender_sockaddr.sa.sa_family == AF_INET ||
 	  notice->z_sender_sockaddr.sa.sa_family == AF_INET6))
 	notice->z_sender_sockaddr.sa.sa_family = AF_INET; /* \/\/hatever *//*XXX*/
 
-    if (!notice->z_class)
-	    notice->z_class = "";
-
-    if (!notice->z_class_inst)
-	    notice->z_class_inst = "";
-
-    if (!notice->z_opcode)
-	    notice->z_opcode = "";
-
-    if (!notice->z_recipient)
-	    notice->z_recipient = "";
-
-    if (!notice->z_default_format)
-	    notice->z_default_format = "";
-
-    ptr = buffer;
-    end = buffer+buffer_len;
-
-    if (buffer_len < strlen(notice->z_version)+1)
-	return (ZERR_HEADERLEN);
-
-    (void) strcpy(ptr, notice->z_version);
-    ptr += strlen(ptr)+1;
-
-    if (ZMakeAscii32(ptr, end-ptr, Z_NUMFIELDS + notice->z_num_other_fields)
-	== ZERR_FIELDLEN)
-	return (ZERR_HEADERLEN);
-    ptr += strlen(ptr)+1;
-
-    if (ZMakeAscii32(ptr, end-ptr, notice->z_kind) == ZERR_FIELDLEN)
-	return (ZERR_HEADERLEN);
-    ptr += strlen(ptr)+1;
-
-    if (ZMakeAscii(ptr, end-ptr, (unsigned char *)&notice->z_uid,
-		   sizeof(ZUnique_Id_t)) == ZERR_FIELDLEN)
-	return (ZERR_HEADERLEN);
-    ptr += strlen(ptr)+1;
-
-    if (ZMakeAscii16(ptr, end-ptr, ntohs(notice->z_port)) == ZERR_FIELDLEN)
-        return (ZERR_HEADERLEN);
-    ptr += strlen(ptr)+1;
-
-    if (ZMakeAscii32(ptr, end-ptr, notice->z_auth) == ZERR_FIELDLEN)
-	return (ZERR_HEADERLEN);
-    ptr += strlen(ptr)+1;
-
-    if (ZMakeAscii32(ptr, end-ptr, notice->z_authent_len) == ZERR_FIELDLEN)
-	return (ZERR_HEADERLEN);
-    ptr += strlen(ptr)+1;
-
-    if (Z_AddField(&ptr, notice->z_ascii_authent, end))
-	return (ZERR_HEADERLEN);
-    if (Z_AddField(&ptr, notice->z_class, end))
-	return (ZERR_HEADERLEN);
-    if (Z_AddField(&ptr, notice->z_class_inst, end))
-	return (ZERR_HEADERLEN);
-    if (Z_AddField(&ptr, notice->z_opcode, end))
-	return (ZERR_HEADERLEN);
-    if (Z_AddField(&ptr, notice->z_sender, end))
-	return (ZERR_HEADERLEN);
-    if (strchr(notice->z_recipient, '@') || !*notice->z_recipient) {
-	if (Z_AddField(&ptr, notice->z_recipient, end))
-	    return (ZERR_HEADERLEN);
-    }
-    else {
-	if (strlen(notice->z_recipient) + strlen(__Zephyr_realm) + 2 >
-	    sizeof(newrecip))
-	    return (ZERR_HEADERLEN);
-	(void) sprintf(newrecip, "%s@%s", notice->z_recipient, __Zephyr_realm);
-	if (Z_AddField(&ptr, newrecip, end))
-	    return (ZERR_HEADERLEN);
-    }
-    if (Z_AddField(&ptr, notice->z_default_format, end))
-	return (ZERR_HEADERLEN);
-
-    /* copy back the end pointer location for crypto checksum */
-    if (cstart)
-	*cstart = ptr;
-    if (ZMakeAscii32(ptr, end-ptr, notice->z_checksum) == ZERR_FIELDLEN)
-        return (ZERR_HEADERLEN);
-    ptr += strlen(ptr)+1;
-    if (cend)
-	*cend = ptr;
-
-    if (Z_AddField(&ptr, notice->z_multinotice, end))
-	return (ZERR_HEADERLEN);
-
-    if (ZMakeAscii(ptr, end-ptr, (unsigned char *)&notice->z_multiuid,
-		   sizeof(ZUnique_Id_t)) == ZERR_FIELDLEN)
-	return (ZERR_HEADERLEN);
-    ptr += strlen(ptr)+1;
-
-    if (notice->z_sender_sockaddr.sa.sa_family == AF_INET) {
-	addrlen = sizeof(notice->z_sender_sockaddr.ip4.sin_addr);
-	addraddr = (unsigned char *)&notice->z_sender_sockaddr.ip4.sin_addr;
-	if (ZMakeAscii(ptr, end - ptr, addraddr, addrlen) == ZERR_FIELDLEN)
-	    return ZERR_HEADERLEN;
-    } else if (notice->z_sender_sockaddr.sa.sa_family == AF_INET6) {
-	addrlen = sizeof(notice->z_sender_sockaddr.ip6.sin6_addr);
-	addraddr = (unsigned char *)&notice->z_sender_sockaddr.ip6.sin6_addr;
-	if (ZMakeZcode(ptr, end - ptr, addraddr, addrlen) == ZERR_FIELDLEN)
-	    return ZERR_HEADERLEN;
-    }
-    ptr += strlen(ptr) + 1;
-
-    if (ZMakeAscii16(ptr, end-ptr, ntohs(notice->z_charset)) == ZERR_FIELDLEN)
-	return ZERR_HEADERLEN;
-    ptr += strlen(ptr) + 1;
-
-    for (i=0;i<notice->z_num_other_fields;i++)
-	if (Z_AddField(&ptr, notice->z_other_fields[i], end))
-	    return (ZERR_HEADERLEN);
-
-    *len = ptr-buffer;
-
-    return (ZERR_NONE);
+    return Z_ZcodeFormatRawHeader(notice, buffer, buffer_len, len,
+				  NULL, NULL, cstart, cend, 0, 0);
 }
 
 static int
@@ -1268,9 +1170,9 @@ Z_SendFragmentedNotice(ZNotice_t *notice,
 Code_t Z_XmitFragment(ZNotice_t *notice,
 		      char *buf,
 		      int len,
-		      int wait)
+		      int waitforack)
 {
-    return(ZSendPacket(buf, len, wait));
+    return(ZSendPacket(buf, len, waitforack));
 }
 
 /* For debugging printing */
@@ -1399,14 +1301,13 @@ Z_InsertZcodeChecksum(krb5_keyblock *keyblock,
      int cksum1_len;  /* length of part after checksum */
      krb5_data cksumbuf;
      krb5_data cksum;
-     unsigned char *key_data;
-     int key_len;
+     unsigned char *cksum_data;
+     unsigned int cksum_data_len;
+     char *cksum_out_data;
      krb5_enctype enctype;
      krb5_cksumtype cksumtype;
      Code_t result;
 
-     key_data = Z_keydata(keyblock);
-     key_len = Z_keylen(keyblock);
      result = Z_ExtractEncCksum(keyblock, &enctype, &cksumtype);
      if (result)
           return (ZAUTH_FAILED);
@@ -1420,19 +1321,22 @@ Z_InsertZcodeChecksum(krb5_keyblock *keyblock,
      cksumbuf.data = malloc(cksumbuf.length);
      if (!cksumbuf.data)
           return ENOMEM;
-     memcpy(cksumbuf.data, cksum_start, cksum0_len);
-     memcpy(cksumbuf.data + cksum0_len, cend, cksum1_len);
-     memcpy(cksumbuf.data + cksum0_len + cksum1_len,
+     cksum_data = (unsigned char *)cksumbuf.data;
+     memcpy(cksum_data, cksum_start, cksum0_len);
+     memcpy(cksum_data + cksum0_len, cend, cksum1_len);
+     memcpy(cksum_data + cksum0_len + cksum1_len,
             notice->z_message, notice->z_message_len);
      /* compute the checksum */
      result = Z_Checksum(&cksumbuf, keyblock, cksumtype,
 			 from_server ? Z_KEYUSAGE_SRV_CKSUM
 			  : Z_KEYUSAGE_CLT_CKSUM,
-                        (char **)&cksum.data, &cksum.length);
+			  &cksum_out_data, &cksum_data_len);
      if (result) {
           free(cksumbuf.data);
           return result;
      }
+     cksum.data = cksum_out_data;
+     cksum.length = cksum_data_len;
 
      /*
       * OK....  we can zcode to a space starting at 'cstart',
@@ -1446,7 +1350,7 @@ Z_InsertZcodeChecksum(krb5_keyblock *keyblock,
      free(cksum.data);
      if (!result) {
           int zcode_len = strlen(cstart) + 1;
-          memcpy(cstart + zcode_len, cksumbuf.data + cksum0_len, cksum1_len);
+          memcpy(cstart + zcode_len, cksum_data + cksum0_len, cksum1_len);
           *length_adjust = zcode_len - cksum_len + (cksum0_len + cksum1_len);
      }
      free(cksumbuf.data);
@@ -1480,7 +1384,6 @@ Z_krb5_verify_cksum(krb5_keyblock *keyblock,
 #else
     krb5_crypto cryptctx;
     Checksum checksum;
-    size_t xlen;
 #endif
 
     memset(&checksum, 0, sizeof(checksum));
@@ -1503,7 +1406,7 @@ Z_krb5_verify_cksum(krb5_keyblock *keyblock,
 
     result = krb5_crypto_init(Z_krb5_ctx, keyblock, keyblock->keytype, &cryptctx);
     if (result)
-	return result;
+	return 0;
 
     /* HOLDING: cryptctx */
     result = krb5_verify_checksum(Z_krb5_ctx, cryptctx, cksumusage,
