@@ -16,9 +16,9 @@ import ctypes.util
 import time
 from ctypes import c_int, c_char, POINTER, c_char_p, sizeof
 
-from zephyr import *
+from zephyr_ctypes import *
 
-__revision__ = "$Id$"
+__revision__ = "$Id: d8a0aca27e6979c9ea682fe654628b557a72c398 $"
 try:
     __version__ = "%s/%s" % (__revision__.split()[3], __revision__.split()[2])
 except IndexError:
