@@ -3,7 +3,7 @@
  *
  *	Created by:	John T. Kohl
  *
- *	$Id$
+ *	$Id: 0f38be5a39e2098ebf9ff36eb72614894fb01716 $
  *
  *	Copyright (c) 1987 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -15,18 +15,22 @@
 
 #if !defined (lint) && !defined (SABER)
 static const char rcsid_access_c[] =
-    "$Id$";
+    "$Id: 0f38be5a39e2098ebf9ff36eb72614894fb01716 $";
 #endif
 
 /*
  *
  * External routines:
  *
- * int access_check(notice, acl, accesstype)
- *    ZNotice_t *notice;
+ * int access_check(sender, who, acl, accesstype)
+ *    char *sender;
+ *    struct sockaddr_in *who;
  *    Acl *acl;
  *    Access accesstype;
  *
+ * int opstaff_check(sender)
+ *    char *sender;
+ *
  * void access_init();
  *
  * void access_reinit();
@@ -58,6 +62,7 @@ static void access_setup(int first);
 
 int
 access_check(char *sender,
+	     struct sockaddr_in *who,
 	     Acl *acl,
 	     Access accesstype)
 {
@@ -96,10 +101,31 @@ access_check(char *sender,
      */
     retval = acl_load(buf);
     if (retval < 0) {
-	syslog(LOG_DEBUG, "Error in acl_load of %s for %s", buf, sender);
+	syslog(LOG_DEBUG, "Error in acl_load of %s for %s",
+		buf, sender ? sender : "unauth client");
 	return 0;
     }
-    return acl_check(buf, sender);
+    return acl_check(buf, sender, who);
+}
+
+int
+opstaff_check(char *sender)
+{
+    char buf[1024];	/* holds the real acl name */
+    int retval;
+
+    snprintf(buf, sizeof buf, "%s/opstaff.acl", acl_dir);
+    /*
+     * If we can't load it (because it probably doesn't exist),
+     * we deny access.
+     */
+    retval = acl_load(buf);
+    if (retval < 0) {
+	syslog(LOG_DEBUG, "Error in acl_load of %s for %s",
+		buf, sender ? sender : "unauth client");
+	return 0;
+    }
+    return acl_check(buf, sender, NULL);
 }
 
 static void
