@@ -6,7 +6,7 @@
  *
  * Created by Ken Raeburn.
  *
- * $Id$
+ * $Id: 910770fb0cf71b3e4acb323d05b0b2795c05adcf $
  *
  * Copyright (c) 1990 by the Massachusetts Institute of Technology.
  * For copying and distribution information, see the file
@@ -36,7 +36,3 @@ void access_reinit(void);
 
 /* found in acl_files.c */
 int acl_load(char *);
-
-/* external data relevant */
-extern int zdebug;
-
