@@ -3,7 +3,7 @@
  *
  *	Created by:	John T. Kohl
  *
- *	$Id$
+ *	$Id: 84c5a70628e77da5d5091ab87b17d5c4c582d765 $
  *
  *	Copyright (c) 1987 by the Massachusetts Institute of Technology.
  *	For copying and distribution information, see the file
@@ -16,7 +16,7 @@
 #define	__ACL__
 
 int acl_add(char *, char *);
-int acl_check(char *, char *);
+int acl_check(char *, char *, struct sockaddr_in *);
 int acl_delete(char *, char *);
 int acl_initialize(char *, int);
 void acl_cache_reset(void);
