@@ -4,7 +4,7 @@
  *	Created by:	John T. Kohl
  *
  *	$Source$
- *	$Id$
+ *	$Id: 344adcf49f37b6d2464b2de8ca1900b397af6892 $
  *	$Author$
  *
  *	Copyright (c) 1987,1988,1991 by the Massachusetts Institute of Technology.
@@ -17,7 +17,7 @@
 #include <sys/socket.h>
 
 #ifndef lint
-static const char rcsid_bdump_c[] = "$Id$";
+static const char rcsid_bdump_c[] = "$Id: 344adcf49f37b6d2464b2de8ca1900b397af6892 $";
 #endif /* lint */
 
 #ifndef MIN
@@ -101,11 +101,10 @@ static struct sockaddr_in bdump_sin;
 #ifdef HAVE_KRB5
 static krb5_auth_context bdump_ac;
 #endif
+static Z_AuthProc bdump_auth_proc;
 
 int bdumping;
 int bdump_concurrent;
-extern char *bdump_version;
-extern int bdump_auth_proto;
 
 /*
  * Functions for performing a brain dump between servers.
@@ -200,7 +199,7 @@ bdump_offer(struct sockaddr_in *who)
 
     /* myname is the hostname */
     /* the class instance is the version number, here it is */
-    /* bdump_version, which is set in main */
+    /* bdump_version, which is set in global.c */
     send_list(ACKED, srv_addr.sin_port, ZEPHYR_ADMIN_CLASS, bdump_version,
 	      ADMIN_BDUMP, myname, "", lyst, 2);
 
@@ -286,6 +285,7 @@ bdump_send(void)
 
     bdumping = 1;
     server->dumping = 1;
+    bdump_auth_proc = ZNOAUTH;
 
     if (bdump_socket >= 0) {
 	/* shut down the listening socket and the timer. */
@@ -406,12 +406,13 @@ bdump_send(void)
 #endif  /* HAVE_KRB5 */
 #ifdef HAVE_KRB4
     case 4:
+	bdump_auth_proc = Z_FormatAuthHeaderWithASCIIAddress;
 	/* here to krb_rd_req from GetKerberosData candidate for refactoring
 	   back into kstuff.c */
 	(void) strcpy(instance, "*"); 		/* let Kerberos fill it in */
 
 	ticket.length = len;
-	memcpy(&ticket.dat, data, MIN(len, sizeof(ticket.dat)));
+	memcpy(&ticket.dat, data, MIN(len, (int)sizeof(ticket.dat)));
 	retval = krb_rd_req(&ticket, SERVER_SERVICE, instance,
 			    from.sin_addr.s_addr, &kdata, srvtab_file);
 	/*
@@ -528,6 +529,7 @@ bdump_get_v12 (ZNotice_t *notice,
 
     bdumping = 1;
     server->dumping = 1;
+    bdump_auth_proc = ZNOAUTH;
 
 #ifdef _POSIX_VERSION
     action.sa_flags = 0;
@@ -709,6 +711,7 @@ bdump_get_v12 (ZNotice_t *notice,
 #endif
 #ifdef HAVE_KRB4
     case 4:
+	bdump_auth_proc = Z_FormatAuthHeaderWithASCIIAddress;
 	/* send an authenticator */
 	retval = SendKerberosData(live_socket, &ticket, SERVER_SERVICE,
 				  SERVER_INSTANCE);
@@ -858,7 +861,8 @@ transmit_tcp(char *pack, int packlen)
 	    syslog(LOG_WARNING, "transmit_tcp: writing length: %m");
 	    retval = errno;
 	} else {
-	    syslog(LOG_WARNING, "transmit_tcp: writing length: %d vs %d", sizeof(length), count);
+	    syslog(LOG_WARNING, "transmit_tcp: writing length: %lu vs %d",
+		   (unsigned long)sizeof(length), count);
 	    retval = ZSRV_LEN;
 	}
 	goto cleanup;
@@ -919,7 +923,8 @@ bdump_send_list_tcp(ZNotice_Kind_t kind,
     if (addr)
 	notice.z_sender_sockaddr.ip4 = *addr; /*XXX*/
 
-    retval = ZFormatNoticeList(&notice, lyst, num, &pack, &packlen, ZNOAUTH);
+    retval = ZFormatNoticeList(&notice, lyst, num, &pack, &packlen,
+			       bdump_auth_proc);
     if (retval != ZERR_NONE) {
 	syslog(LOG_ERR, "bdump_send_list_tcp: ZFormatNotice: %s", error_message(retval));
 	return retval;
@@ -965,9 +970,10 @@ cleanup(Server *server)
     zdbug((LOG_DEBUG, "bdump cleanup"));
 
     if (server != limbo_server) {
-	server->state = SERV_DEAD;
+	if (server->state != SERV_STARTING)
+	    server->state = SERV_DEAD;
 	timer_reset(server->timer);
-	server->timer = timer_set_rel(0L, server_timo, server);
+	server->timer = timer_set_rel(server->timeout, server_timo, server);
     }
     shutdown_file_pointers ();
 #ifdef _POSIX_VERSION
@@ -1177,6 +1183,8 @@ bdump_recv_loop(Server *server)
     Client *client = NULL;
     struct sockaddr_in who;
 #ifdef HAVE_KRB5
+    uint32_t client_enctype;
+    uint32_t client_keysize;
     unsigned char buf[512];
     int blen;
 #endif
@@ -1318,9 +1326,11 @@ bdump_recv_loop(Server *server)
 			syslog(LOG_ERR,"bdump_recv_loop: bad keyblock read: %s (%s)",
 			       error_message(retval), cp);
 		    } else {
+			memcpy(&client_enctype, &buf[0], sizeof(uint32_t));
+			memcpy(&client_keysize, &buf[4], sizeof(uint32_t));
 			retval = Z_krb5_init_keyblock(Z_krb5_ctx,
-						    ntohl(*(krb5_enctype *)&buf[0]),
-						    ntohl(*(uint32_t *)&buf[4]),
+						    ntohl(client_enctype),
+						    ntohl(client_keysize),
 						    &client->session_keyblock);
 			if (retval) {
 			    syslog(LOG_ERR, "bdump_recv_loop: failed to allocate keyblock: %s",
@@ -1506,7 +1516,7 @@ send_normal_tcp(ZNotice_Kind_t kind,
     notice.z_message_len = len;
     notice.z_num_other_fields = 0;
 
-    retval = ZFormatNotice(&notice, &pack, &packlen, ZNOAUTH);
+    retval = ZFormatNotice(&notice, &pack, &packlen, bdump_auth_proc);
     if (retval != ZERR_NONE) {
 	syslog(LOG_WARNING, "send_normal_tcp: ZFormatNotice: %s", error_message(retval));
 	return retval;
@@ -1533,12 +1543,12 @@ get_packet(char **packet, int *buflen, int *retlen)
     char *p;
 
     result = net_read(input, (char *) &length, sizeof(unsigned short));
-    if (result < sizeof(short)) {
+    if (result < (int)sizeof(short)) {
 	if (result < 0) {
 	    return errno;
 	} else {
-	    syslog(LOG_ERR, "get_packet: received length: %d vs %d (%m)", result,
-		   sizeof(short));
+	    syslog(LOG_ERR, "get_packet: received length: %d vs %lu (%m)",
+		   result, (unsigned long)sizeof(short));
 	    return ZSRV_LEN;
 	}
     }
