@@ -18,14 +18,14 @@
 #ifndef lint
 #ifndef SABER
 static const char rcsid_dispatch_c[] =
-"$Id$";
+"$Id: ea95d3afb22de6554bb53146db3a703d8b17c264 $";
 #endif
 #endif
 
 enum {
     NACKTAB_HASHSIZE = 1023
 };
-inline unsigned int
+inline static unsigned int
 nacktab_hashval(struct sockaddr_in sa, ZUnique_Id_t uid) {
     return (sa.sin_addr.s_addr ^
 	    sa.sin_port ^
@@ -36,11 +36,6 @@ nacktab_hashval(struct sockaddr_in sa, ZUnique_Id_t uid) {
 
 #define HOSTS_SIZE_INIT			256
 
-#ifdef DEBUG
-const char *ZNoticeKinds[9] = {"UNSAFE", "UNACKED", "ACKED", "HMACK",
-				    "HMCTL", "SERVACK", "SERVNAK", "CLIENTACK",
-				    "STAT"};
-#endif
 /*
  *
  * External Routines:
@@ -205,6 +200,12 @@ handle_packet(void)
 	whence = &whoisit;
     }
 
+    /* Don't bother checking authentication on client ACKs */
+    if (new_notice.z_kind == CLIENTACK) {
+	nack_cancel(&new_notice, &whoisit);
+	return;
+    }
+
     /* Clients don't check auth of acks, nor do we make it so they
        can in general, so this is safe. */
     if (new_notice.z_kind == SERVACK || new_notice.z_kind == SERVNAK) {
@@ -230,13 +231,9 @@ dispatch(ZNotice_t *notice,
 {
     Code_t status;
     String *notice_class;
-    struct sockaddr_in who2;
     int authflag;
     ZRealm *realm;
     char *cp;
-#ifdef DEBUG
-    char dbg_buf[BUFSIZ];
-#endif
 
     authflag = (auth == ZAUTH_YES);
 
@@ -252,8 +249,6 @@ dispatch(ZNotice_t *notice,
 	return;
     }
 
-    who2 = *who;
-
     notice_class = make_string(notice->z_class,1);
 
     if (from_server) {
@@ -311,7 +306,7 @@ sendit(ZNotice_t *notice,
        int external)
 {
     static int send_counter = 0;
-    char recipbuf[MAX_PRINCIPAL_SIZE], *recipp;
+    char recipbuf[MAX_PRINCIPAL_SIZE], *recipp, *acl_sender;
     int any = 0;
     Acl *acl;
     Destination dest;
@@ -323,14 +318,7 @@ sendit(ZNotice_t *notice,
 
       acl = class_get_acl(class);
       if (acl != NULL) {
-	/* if controlled and not auth, fail */
-        if (!auth) {
-            syslog(LOG_WARNING, "sendit unauthentic %s from %s",
-                   notice->z_class, notice->z_sender);
-	    clt_ack(notice, who, AUTH_FAILED);
-            free_string(class);
-            return;
-        }
+	acl_sender = auth ? notice->z_sender : 0;
 	/* if from foreign realm server, disallow if not realm of sender */
 	rlm = realm_which_realm(who);
 	if (rlm) {
@@ -343,17 +331,18 @@ sendit(ZNotice_t *notice,
 	  }
 	}
 	/* if not auth to transmit, fail */
-	if (!access_check(notice->z_sender, acl, TRANSMIT)) {
-	    syslog(LOG_WARNING, "sendit unauthorized %s from %s",
-		   notice->z_class, notice->z_sender);
+	if (!access_check(acl_sender, who, acl, TRANSMIT)) {
+	    syslog(LOG_WARNING, "sendit unauthorized %s from %s%s",
+		   notice->z_class, notice->z_sender, auth ? "" : " (unauth)");
 	    clt_ack(notice, who, AUTH_FAILED);
 	    free_string(class);
 	    return;
 	}
 	/* sender != inst and not auth to send to others --> fail */
 	if (strcmp(notice->z_sender, notice->z_class_inst) != 0 &&
-	    !access_check(notice->z_sender, acl, INSTUID)) {
-	    syslog(LOG_WARNING, "sendit unauth uid %s %s.%s", notice->z_sender,
+	    !access_check(acl_sender, who, acl, INSTUID)) {
+	    syslog(LOG_WARNING, "sendit unauth uid %s%s %s.%s",
+		   notice->z_sender, auth ? "" : " (unauth)",
 		   notice->z_class, notice->z_class_inst);
 	    clt_ack(notice, who, AUTH_FAILED);
 	    free_string(class);
@@ -687,12 +676,13 @@ xmit(ZNotice_t *notice,
     }
 
     /* now we've sent it, mark it as not ack'ed */
-    if (!retval)
+    if (!retval) {
 	nacked = (Unacked *) malloc(sizeof(Unacked));
-    if (!nacked) {
-	/* no space: just punt */
-	syslog(LOG_WARNING, "xmit nack malloc");
-	retval = ENOMEM;
+	if (!nacked) {
+	    /* no space: just punt */
+	    syslog(LOG_WARNING, "xmit nack malloc");
+	    retval = ENOMEM;
+	}
     }
 
     if (!retval) {
@@ -783,7 +773,6 @@ clt_ack(ZNotice_t *notice,
     ZNotice_t acknotice;
     ZPacket_t ackpack;
     int packlen;
-    char *sent_name;
     Code_t retval;
 
     if (bdumping) {		/* don't ack while dumping */
@@ -797,21 +786,17 @@ clt_ack(ZNotice_t *notice,
     switch (sent) {
       case SENT:
 	acknotice.z_message = ZSRVACK_SENT;
-	sent_name = "sent";
 	break;
       case NOT_FOUND:
 	acknotice.z_message = ZSRVACK_FAIL;
 	acknotice.z_kind = SERVNAK;
-	sent_name = "fail";
 	break;
       case AUTH_FAILED:
 	acknotice.z_kind = SERVNAK;
 	acknotice.z_message = ZSRVACK_NOTSENT;
-	sent_name = "nak/not_sent";
 	break;
       case NOT_SENT:
 	acknotice.z_message = ZSRVACK_NOTSENT;
-	sent_name = "not_sent";
 	break;
       default:
 	abort ();
@@ -889,11 +874,6 @@ nack_cancel(ZNotice_t *notice,
 	   notice->z_uid.tv.tv_sec, notice->z_uid.tv.tv_usec));
 }
 
-/* for compatibility when sending subscription information to old clients */
-#ifdef OLD_COMPAT
-#define	OLD_ZEPHYR_VERSION	"ZEPH0.0"
-#endif /* OLD_COMPAT */
-
 /* Dispatch an HM_CTL notice. */
 
 Code_t
@@ -903,7 +883,7 @@ hostm_dispatch(ZNotice_t *notice,
 	       Server *server)
 {
     char *opcode = notice->z_opcode;
-    int i, add = 0, remove = 0;
+    int i, add_it = 0, remove_it = 0;
 
     if (notice->z_kind == HMACK) {
 	/* Ignore. */
@@ -919,23 +899,23 @@ hostm_dispatch(ZNotice_t *notice,
 	if (server == me_server) {
 	    server_forward(notice, auth, who);
 	    ack(notice, who);
-	    add = 1;
+	    add_it = 1;
 	}
     } else if (strcmp(opcode, HM_ATTACH) == 0) {
 	if (server == me_server) {
 	    server_forward(notice, auth, who);
 	    ack(notice, who);
-	    add = 1;
+	    add_it = 1;
 	} else {
-	    remove = 1;
+	    remove_it = 1;
 	}
     } else if (strcmp(opcode, HM_DETACH) == 0) {
-	remove = 1;
+	remove_it = 1;
     } else {
 	syslog(LOG_WARNING, "hm_dispatch: unknown opcode %s", opcode);
     }
 
-    if (add) {
+    if (add_it) {
 	for (i = 0; i < num_hosts; i++) {
 	    if (hosts[i].s_addr == who->sin_addr.s_addr)
 		break;
@@ -956,7 +936,7 @@ hostm_dispatch(ZNotice_t *notice,
 	    }
 	    hosts[num_hosts++] = who->sin_addr;
 	}
-    } else if (remove) {
+    } else if (remove_it) {
 	for (i = 0; i < num_hosts; i++) {
 	    if (hosts[i].s_addr == who->sin_addr.s_addr) {
 		memmove(&hosts[i], &hosts[i + 1], num_hosts - (i + 1));
@@ -978,7 +958,7 @@ control_dispatch(ZNotice_t *notice,
 		 struct sockaddr_in *who,
 		 Server *server)
 {
-    char *opcode = notice->z_opcode;
+    char *target, *opcode = notice->z_opcode;
     Client *client;
     Code_t retval;
     int wantdefs;
@@ -1008,17 +988,12 @@ control_dispatch(ZNotice_t *notice,
 	   someone who has no subscriptions does NOT get a SERVNAK
 	   but rather an empty list.  Note we must therefore
 	   check authentication inside subscr_sendlist */
-#ifdef OLD_COMPAT
-	/* only acknowledge if *not* old version; the old version
-	   acknowledges the packet with the reply */
-	if (strcmp(notice->z_version, OLD_ZEPHYR_VERSION) != 0)
-	    ack(notice, who);
-#else /* !OLD_COMPAT */
 	ack(notice, who);
-#endif /* OLD_COMPAT */
 	subscr_sendlist(notice, auth, who);
 	return ZERR_NONE;
     } else if (!auth) {
+        syslog(LOG_INFO, "unauthenticated %s message purportedly from %s",
+               opcode, notice->z_sender);
 	if (server == me_server)
 	    clt_ack(notice, who, AUTH_FAILED);
 	return ZERR_NONE;
@@ -1042,6 +1017,10 @@ control_dispatch(ZNotice_t *notice,
 	}
 	if (strcmp(client->principal->string, notice->z_sender) != 0) {
 	    /* you may only subscribe for your own clients */
+            syslog(LOG_NOTICE,
+                   "subscr request from %s on port used by %s (%s.%d)",
+                   notice->z_sender, client->principal->string,
+                   inet_ntoa(who->sin_addr), ntohs(notice->z_port)); /* XXX */
 	    if (server == me_server)
 		clt_ack(notice, who, AUTH_FAILED);
 	    return ZERR_NONE;
@@ -1080,6 +1059,10 @@ control_dispatch(ZNotice_t *notice,
 	if (client != NULL) {
 	    if (strcmp(client->principal->string, notice->z_sender) != 0) {
 		/* you may only cancel for your own clients */
+                syslog(LOG_NOTICE,
+                       "unsubscr request from %s on port used by %s (%s.%d)",
+                       notice->z_sender, client->principal->string,
+                       inet_ntoa(who->sin_addr), ntohs(notice->z_port)); /*XXX*/
 		if (server == me_server)
 		    clt_ack(notice, who, AUTH_FAILED);
 		return ZERR_NONE;
@@ -1099,12 +1082,36 @@ control_dispatch(ZNotice_t *notice,
 	}
 	if (strcmp(client->principal->string, notice->z_sender) != 0) {
 	    /* you may only cancel for your own clients */
+            syslog(LOG_NOTICE,
+                   "cancel request from %s on port used by %s (%s.%d)",
+                   notice->z_sender, client->principal->string,
+                   inet_ntoa(who->sin_addr), ntohs(notice->z_port)); /* XXX */
 	    if (server == me_server)
 		clt_ack(notice, who, AUTH_FAILED);
 	    return ZERR_NONE;
 	}
 	/* don't flush locations here, let him do it explicitly */
 	client_deregister(client, 0);
+    } else if (strcmp(opcode, CLIENT_FLUSHSUBS) == 0) {
+	target = notice->z_sender;
+	if (notice->z_message_len > 0 && *notice->z_message != 0) {
+	    target = notice->z_message;
+	    if (memchr(target, '\0', notice->z_message_len) == NULL) {
+		syslog(LOG_WARNING, "malformed flushsubs");
+		if (server == me_server)
+		    nack(notice, who);
+		return ZERR_NONE;
+	    }
+	    if (strcmp(target, notice->z_sender) != 0 &&
+		!opstaff_check(notice->z_sender)) {
+		syslog(LOG_NOTICE, "unauth flushsubs for %s by %s",
+		       target, notice->z_sender);
+		if (server == me_server)
+		    clt_ack(notice, who, AUTH_FAILED);
+		return ZERR_NONE;
+	    }
+	}
+	client_flush_princ(target);
     } else {
 	syslog(LOG_WARNING, "unknown ctl opcode %s", opcode);
 	if (server == me_server) {
