@@ -16,7 +16,7 @@
 
 #ifndef lint
 #ifndef SABER
-static const char rcsid_kstuff_c[] = "$Id$";
+static const char rcsid_kstuff_c[] = "$Id: bafd64801d2feb883fae9ed9588b361a8b5d912c $";
 #endif
 #endif
 
@@ -45,7 +45,7 @@ GetKerberosData(int fd, /* file descr. to read from */
 {
     char p[20];
     KTEXT_ST ticket;		/* will get Kerberos ticket from client */
-    int i;
+    unsigned int i;
     char instance[INST_SZ];
 
     /*
@@ -105,8 +105,8 @@ SendKerberosData(int fd,	/* file descriptor to write onto */
 {
     int rem;
     char p[32];
-    int written;
-    int size_to_write;
+    size_t written;
+    size_t size_to_write;
 
     rem = krb_mk_req(ticket, service, host, (char *)ZGetRealm(), (u_long) 0);
     if (rem != KSUCCESS)
@@ -115,10 +115,10 @@ SendKerberosData(int fd,	/* file descriptor to write onto */
     (void) sprintf(p,"%d ",ticket->length);
     size_to_write = strlen (p);
     if ((written = write(fd, p, size_to_write)) != size_to_write)
-	return (written < 0) ? errno : ZSRV_PKSHORT;
+	return ((ssize_t)written < 0) ? errno : ZSRV_PKSHORT;
     if ((written = write(fd, ticket->dat, ticket->length))
 	!= ticket->length)
-	return (written < 0) ? errno : ZSRV_PKSHORT;
+	return ((ssize_t)written < 0) ? errno : ZSRV_PKSHORT;
 
     return 0;
 }
@@ -190,7 +190,7 @@ ReadKerberosData(int fd, int *size, char **data, int *proto) {
 Code_t
 GetKrb5Data(int fd, krb5_data *data) {
     char p[20];
-    int i;
+    unsigned int i;
     char *dst;
 
     for (i=0; i<20; i++) {
@@ -229,12 +229,12 @@ GetKrb5Data(int fd, krb5_data *data) {
 Code_t
 SendKrb5Data(int fd, krb5_data *data) {
     char p[32];
-    int written, size_to_write;
-    sprintf(p, "V5-%d ", data->length);
+    size_t written, size_to_write;
+    sprintf(p, "V5-%lu ", (unsigned long)data->length);
     size_to_write = strlen (p);
     if (size_to_write != (written = write(fd, p, size_to_write)) ||
         data->length != (written = write(fd, data->data, data->length))) {
-        return (written < 0) ? errno : ZSRV_PKSHORT;
+        return ((ssize_t)written < 0) ? errno : ZSRV_PKSHORT;
     }
     return 0;
 }
@@ -262,9 +262,9 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
     int valid;
     char *cksum0_base, *cksum1_base = NULL, *cksum2_base;
     char *x;
-    unsigned char *asn1_data, *key_data;
+    unsigned char *asn1_data, *key_data, *cksum_data;
     int asn1_len, key_len, cksum0_len = 0, cksum1_len = 0, cksum2_len = 0;
-    krb5_flags acflags;
+    KRB5_AUTH_CON_FLAGS_TYPE acflags;
 #ifdef KRB5_AUTH_CON_GETAUTHENTICATOR_TAKES_DOUBLE_POINTER
     krb5_authenticator *authenticator;
 #define KRB5AUTHENT authenticator
@@ -280,8 +280,10 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
         return ZAUTH_NO;
 
     /* Check for bogus authentication data length. */
-    if (notice->z_authent_len <= 0)
+    if (notice->z_authent_len <= 0) {
+        syslog(LOG_DEBUG, "ZCheckSrvAuthentication: bogus authenticator length");
         return ZAUTH_FAILED;
+    }
 
 #ifdef HAVE_KRB4
     if (notice->z_ascii_authent[0] != 'Z' && realm == NULL)
@@ -295,6 +297,7 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
     if (ZReadZcode((unsigned char *)notice->z_ascii_authent,
                    authbuf,
                    len, &len) == ZERR_BADFIELD) {
+        syslog(LOG_DEBUG, "ZCheckSrvAuthentication: ZReadZcode: Improperly formatted field");
         return ZAUTH_FAILED;
     }
 
@@ -312,8 +315,9 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
     result = krb5_kt_resolve(Z_krb5_ctx,
                         keytab_file, &keytabid);
     if (result) {
-      free(authbuf);
-      return ZAUTH_FAILED;
+        free(authbuf);
+        syslog(LOG_DEBUG, "ZCheckSrvAuthentication: krb5_kt_resolve: %s", error_message(result));
+        return ZAUTH_FAILED;
     }
 
     /* HOLDING: authbuf, keytabid */
@@ -322,13 +326,17 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
     if (result) {
         krb5_kt_close(Z_krb5_ctx, keytabid);
         free(authbuf);
+        syslog(LOG_DEBUG, "ZCheckSrvAuthentication: krb5_auth_con_init: %s", error_message(result));
         return ZAUTH_FAILED;
     }
 
+    /* HOLDING: authbuf, keytabid, authctx */
     result = krb5_auth_con_getflags(Z_krb5_ctx, authctx, &acflags);
     if (result) {
+        krb5_auth_con_free(Z_krb5_ctx, authctx);
         krb5_kt_close(Z_krb5_ctx, keytabid);
         free(authbuf);
+        syslog(LOG_DEBUG, "ZCheckSrvAuthentication: krb5_auth_con_getflags: %s", error_message(result));
         return ZAUTH_FAILED;
     }
 
@@ -336,12 +344,13 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
 
     result = krb5_auth_con_setflags(Z_krb5_ctx, authctx, acflags);
     if (result) {
+        krb5_auth_con_free(Z_krb5_ctx, authctx);
         krb5_kt_close(Z_krb5_ctx, keytabid);
         free(authbuf);
+        syslog(LOG_DEBUG, "ZCheckSrvAuthentication: krb5_auth_con_setflags: %s", error_message(result));
         return ZAUTH_FAILED;
     }
 
-    /* HOLDING: authbuf, authctx */
     result = krb5_build_principal(Z_krb5_ctx, &server, strlen(__Zephyr_realm),
 				  __Zephyr_realm, SERVER_SERVICE,
 				  SERVER_INSTANCE, NULL);
@@ -352,13 +361,14 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
     }
     krb5_kt_close(Z_krb5_ctx, keytabid);
 
+    /* HOLDING: authbuf, authctx */
     if (result) {
-      if (result == KRB5KRB_AP_ERR_REPEAT)
-	syslog(LOG_DEBUG, "ZCheckSrvAuthentication: k5 auth failed: %s",
-	       error_message(result));
-      else
-        syslog(LOG_WARNING,"ZCheckSrvAuthentication: k5 auth failed: %s",
-	       error_message(result));
+        if (result == KRB5KRB_AP_ERR_REPEAT)
+            syslog(LOG_DEBUG, "ZCheckSrvAuthentication: k5 auth failed: %s",
+                   error_message(result));
+        else
+            syslog(LOG_WARNING,"ZCheckSrvAuthentication: k5 auth failed: %s",
+                   error_message(result));
         free(authbuf);
         krb5_auth_con_free(Z_krb5_ctx, authctx);
         return ZAUTH_FAILED;
@@ -367,11 +377,12 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
     /* HOLDING: authbuf, authctx, tkt */
 
     if (tkt == 0 || !Z_tktprincp(tkt)) {
-       if (tkt)
-	   krb5_free_ticket(Z_krb5_ctx, tkt);
-       free(authbuf);
-       krb5_auth_con_free(Z_krb5_ctx, authctx);
-       return ZAUTH_FAILED;
+        if (tkt)
+            krb5_free_ticket(Z_krb5_ctx, tkt);
+        free(authbuf);
+        krb5_auth_con_free(Z_krb5_ctx, authctx);
+        syslog(LOG_WARNING, "ZCheckSrvAuthentication: No Ticket");
+        return ZAUTH_FAILED;
     }
 
     princ = Z_tktprinc(tkt);
@@ -380,13 +391,14 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
         krb5_free_ticket(Z_krb5_ctx, tkt);
         free(authbuf);
         krb5_auth_con_free(Z_krb5_ctx, authctx);
+        syslog(LOG_WARNING, "ZCheckSrvAuthentication: No... Ticket?");
         return ZAUTH_FAILED;
     }
 
     /* HOLDING: authbuf, authctx, tkt */
     result = krb5_unparse_name(Z_krb5_ctx, princ, &name);
     if (result) {
-        syslog(LOG_WARNING, "k5 unparse_name failed: %s",
+        syslog(LOG_WARNING, "ZCheckSrvAuthentication: krb5_unparse_name failed: %s",
                error_message(result));
         free(authbuf);
         krb5_auth_con_free(Z_krb5_ctx, authctx);
@@ -398,31 +410,43 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
 
     /* HOLDING: authbuf, authctx, name */
     if (strcmp(name, sender)) {
-        syslog(LOG_WARNING, "k5 name mismatch: '%s' vs '%s'",
+        syslog(LOG_WARNING, "ZCheckSrvAuthentication: name mismatch: '%s' vs '%s'",
                name, sender);
         krb5_auth_con_free(Z_krb5_ctx, authctx);
+#ifdef HAVE_KRB5_FREE_UNPARSED_NAME
+        krb5_free_unparsed_name(Z_krb5_ctx, name);
+#else
         free(name);
+#endif
         free(authbuf);
         return ZAUTH_FAILED;
     }
+#ifdef HAVE_KRB5_FREE_UNPARSED_NAME
+    krb5_free_unparsed_name(Z_krb5_ctx, name);
+#else
     free(name);
+#endif
     free(authbuf);
 
     /* HOLDING: authctx */
     /* Get an authenticator so we can get the keyblock */
     result = krb5_auth_con_getauthenticator (Z_krb5_ctx, authctx,
     					     &authenticator);
-    if(result) {
+    if (result) {
         krb5_auth_con_free(Z_krb5_ctx, authctx);
+        syslog(LOG_WARNING, "ZCheckSrvAuthentication: krb5_auth_con_getauthenticator failed: %s",
+               error_message(result));
         return ZAUTH_FAILED;
     }
 
     /* HOLDING: authctx, authenticator */
     result = krb5_auth_con_getkey(Z_krb5_ctx, authctx, &keyblock);
     if (result) {
-      krb5_auth_con_free(Z_krb5_ctx, authctx);
-      krb5_free_authenticator(Z_krb5_ctx, KRB5AUTHENT);
-      return (ZAUTH_FAILED);
+        krb5_auth_con_free(Z_krb5_ctx, authctx);
+        krb5_free_authenticator(Z_krb5_ctx, KRB5AUTHENT);
+        syslog(LOG_WARNING, "ZCheckSrvAuthentication: krb5_auth_con_getkey failed: %s",
+               error_message(result));
+        return (ZAUTH_FAILED);
     }
 
     /* HOLDING: authctx, authenticator, keyblock */
@@ -434,6 +458,8 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
         krb5_free_keyblock(Z_krb5_ctx, keyblock);
         krb5_auth_con_free(Z_krb5_ctx, authctx);
         krb5_free_authenticator(Z_krb5_ctx, KRB5AUTHENT);
+        syslog(LOG_WARNING, "ZCheckSrvAuthentication: Z_ExtractEncCksum failed: %s",
+               error_message(result));
         return (ZAUTH_FAILED);
     }
     /* HOLDING: authctx, authenticator, keyblock */
@@ -473,8 +499,7 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
 	if (notice->z_num_other_fields)
 	    x = notice->z_other_fields[notice->z_num_other_fields - 1];
 	else {
-	    /* see also ZCheckRealmAuthentication
-	       and lib/ZCkZaut.c:ZCheckZcodeAuthentication  */
+	    /* see lib/ZCkZaut.c:ZCheckZcodeAuthentication  */
 	    /* XXXXXXXXXXXXXXXXXXXXXXX */
 	    if (notice->z_num_hdr_fields > 16)
 		x = cksum1_base + strlen(cksum1_base) + 1; /* multinotice */
@@ -493,14 +518,14 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
     /*XXX we may wish to ditch this code someday?*/
     if ((!notice->z_ascii_checksum || *notice->z_ascii_checksum != 'Z') &&
         key_len == 8 &&
-        (enctype == ENCTYPE_DES_CBC_CRC ||
-         enctype == ENCTYPE_DES_CBC_MD4 ||
-         enctype == ENCTYPE_DES_CBC_MD5)) {
+        (enctype == (krb5_enctype)ENCTYPE_DES_CBC_CRC ||
+         enctype == (krb5_enctype)ENCTYPE_DES_CBC_MD4 ||
+         enctype == (krb5_enctype)ENCTYPE_DES_CBC_MD5)) {
       /* try old-format checksum (covers cksum0 only) */
 
       ZChecksum_t our_checksum;
 
-      if (realm == NULL) 
+      if (realm == NULL)
 	  our_checksum = compute_checksum(notice, key_data);
       else
 	  our_checksum = compute_rlm_checksum(notice, key_data);
@@ -509,10 +534,12 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
       krb5_auth_con_free(Z_krb5_ctx, authctx);
       krb5_free_authenticator(Z_krb5_ctx, KRB5AUTHENT);
 
-      if (our_checksum == notice->z_checksum)
-	return ZAUTH_YES;
-      else
-	return ZAUTH_FAILED;
+      if (our_checksum == notice->z_checksum) {
+          return ZAUTH_YES;
+      } else {
+          syslog(LOG_DEBUG, "ZCheckSrvAuthentication: des quad checksum mismatch");
+          return ZAUTH_FAILED;
+      }
     }
 
     /* HOLDING: authctx, authenticator */
@@ -523,14 +550,16 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
         krb5_free_keyblock(Z_krb5_ctx, keyblock);
         krb5_auth_con_free(Z_krb5_ctx, authctx);
         krb5_free_authenticator(Z_krb5_ctx, KRB5AUTHENT);
+        syslog(LOG_ERR, "ZCheckSrvAuthentication: malloc(cksumbuf.data): %m");
         return ZAUTH_FAILED;
     }
     /* HOLDING: authctx, authenticator, cksumbuf.data */
 
-    memcpy(cksumbuf.data, cksum0_base, cksum0_len);
+    cksum_data = (unsigned char *)cksumbuf.data;
+    memcpy(cksum_data, cksum0_base, cksum0_len);
     if (cksum1_len)
-	memcpy(cksumbuf.data + cksum0_len, cksum1_base, cksum1_len);
-    memcpy(cksumbuf.data + cksum0_len + cksum1_len,
+	memcpy(cksum_data + cksum0_len, cksum1_base, cksum1_len);
+    memcpy(cksum_data + cksum0_len + cksum1_len,
            cksum2_base, cksum2_len);
 
     /* decode zcoded checksum */
@@ -542,6 +571,7 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
         krb5_auth_con_free(Z_krb5_ctx, authctx);
         krb5_free_authenticator(Z_krb5_ctx, KRB5AUTHENT);
         free(cksumbuf.data);
+        syslog(LOG_ERR, "ZCheckSrvAuthentication: malloc(asn1_data): %m");
         return ZAUTH_FAILED;
     }
     /* HOLDING: authctx, authenticator, cksumbuf.data, asn1_data */
@@ -553,6 +583,8 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
         krb5_free_authenticator(Z_krb5_ctx, KRB5AUTHENT);
         free(asn1_data);
         free(cksumbuf.data);
+        syslog(LOG_WARNING, "ZCheckSrvAuthentication: ZReadZcode: %s",
+               error_message(result));
         return ZAUTH_FAILED;
     }
     /* HOLDING: asn1_data, cksumbuf.data, authctx, authenticator */
@@ -573,10 +605,12 @@ ZCheckSrvAuthentication(ZNotice_t *notice,
     krb5_free_keyblock(Z_krb5_ctx, keyblock);
     free(cksumbuf.data);
 
-    if (valid)
-        return (ZAUTH_YES);
-    else
-        return (ZAUTH_FAILED);
+    if (valid) {
+        return ZAUTH_YES;
+    } else {
+        syslog(LOG_DEBUG, "ZCheckSrvAuthentication: Z_krb5_verify_cksum: failed");
+        return ZAUTH_FAILED;
+    }
 #else
     return (notice->z_auth) ? ZAUTH_YES : ZAUTH_NO;
 #endif
@@ -659,10 +693,9 @@ static ZChecksum_t compute_rlm_checksum(ZNotice_t *notice,
 					unsigned char *session_key)
 {
     ZChecksum_t checksum;
-    char *cstart, *cend, *hstart = notice->z_packet;
+    char *cstart, *hstart = notice->z_packet;
 
     cstart = notice->z_default_format + strlen(notice->z_default_format) + 1;
-    cend = cstart + strlen(cstart) + 1;
     checksum = z_quad_cksum((unsigned char *)hstart, NULL,
 			    cstart - hstart, 0, session_key);
 
@@ -708,7 +741,7 @@ ZSetSession(krb5_keyblock *keyblock) {
     }
 
     if (result) /*XXX we're out of memory? */
-	;
+	return;
 }
 #endif
 #ifdef HAVE_KRB4
