@@ -19,7 +19,7 @@
 #ifndef lint
 #ifndef SABER
 static const char rcsid_main_c[] =
-    "$Id$";
+    "$Id: 0186b177f716b3a82745c02e5e4216717d49cf4a $";
 #endif
 #endif
 
@@ -71,88 +71,38 @@ static void detach(void);
 static short doreset = 0;		/* if it becomes 1, perform
 					   reset functions */
 
-int nfds;				/* max file descriptor for select() */
-int srv_socket;				/* dgram socket for clients
-					   and other servers */
-int bdump_socket = -1;			/* brain dump socket fd
-					   (closed most of the time) */
-fd_set interesting;			/* the file descrips we are listening
-					   to right now */
-struct sockaddr_in srv_addr;		/* address of the socket */
+static char *programname;               /* set to the basename of argv[0] */
 
-Unacked *nacklist = NULL;		/* list of packets waiting for ack's */
-
-unsigned short hm_port;			/* host manager receiver port */
-unsigned short hm_srv_port;		/* host manager server sending port */
-
-char *programname;			/* set to the basename of argv[0] */
-char myname[NS_MAXDNAME];		/* my host name */
-
-char list_file[128];
 #ifdef HAVE_KRB5
-char keytab_file[128];
 static char tkt5_file[256];
 #endif
 #ifdef HAVE_KRB4
-char srvtab_file[128];
 static char tkt_file[128];
 #endif
-#if defined(HAVE_KRB4) || defined(HAVE_KRB5)
-char my_realm[REALM_SZ];
-#endif
-char acl_dir[128];
-char subs_file[128];
-
-int zdebug;
-#ifdef DEBUG
-int zalone;
-#endif
-
-struct timeval t_local;			/* store current time for other uses */
 
 static int dump_db_flag = 0;
 static int dump_strings_flag = 0;
 
-u_long npackets;			/* number of packets processed */
-time_t uptime;				/* when we started operations */
 static int nofork;
-struct in_addr my_addr;
-char *bdump_version = "1.2";
 
-#ifdef HAVE_KRB5
-int bdump_auth_proto = 5;
-#else /* HAVE_KRB5 */
-#ifdef HAVE_KRB4
-int bdump_auth_proto = 4;
-#else /* HAVE_KRB4 */
-int bdump_auth_proto = 0;
-#endif /* HAVE_KRB4 */
-#endif /* HAVE_KRB5 */
-
-#ifdef HAVE_KRB5
-krb5_ccache Z_krb5_ccache;
-krb5_keyblock *__Zephyr_keyblock;
-#else
-#ifdef HAVE_KRB4
-C_Block __Zephyr_session;
-#endif
+#if defined(HAVE_KRB4) || defined(HAVE_KRB5)
+static char my_realm[REALM_SZ];
 #endif
 
 int
 main(int argc,
      char **argv)
 {
+    int nselect;		/* #fildes to select on */
     int nfound;			/* #fildes ready on select */
-    fd_set readable;
-    struct timeval tv;
+    fd_set readable, writable;
+    struct timeval tv, *tvp;
     int init_from_dump = 0;
     char *dumpfile;
 #ifdef _POSIX_VERSION
     struct sigaction action;
 #endif
     int optchar;			/* option processing */
-    extern char *optarg;
-    extern int optind;
 
     sprintf(list_file, "%s/zephyr/%s", SYSCONFDIR, SERVER_LIST_FILE);
 #ifdef HAVE_KRB4
@@ -308,8 +258,6 @@ main(int argc,
     gettimeofday(&t_local, NULL);
     uptime = NOW;
 
-    realm_wakeup();
-
     for (;;) {
 	if (doreset)
 	    do_reset();
@@ -322,13 +270,24 @@ main(int argc,
 	timer_process();
 
 	readable = interesting;
+	FD_ZERO(&writable);
+	tvp = timer_timeout(&tv);
+#ifdef HAVE_ARES
+	nselect = ares_fds(achannel, &readable, &writable);
+	if (nselect < nfds)
+	    nselect = nfds;
+	tvp = ares_timeout(achannel, tvp, &tv);
+#else
+	nselect = nfds;
+#endif
 	if (msgs_queued()) {
 	    /* when there is input in the queue, we
 	       artificially set up to pick up the input */
 	    nfound = 1;
 	    FD_ZERO(&readable);
+	    FD_ZERO(&writable);
 	} else  {
-	    nfound = select(nfds, &readable, NULL, NULL, timer_timeout(&tv));
+	    nfound = select(nselect, &readable, &writable, NULL, tvp);
 	}
 
 	/* Initialize t_local for other uses */
@@ -342,6 +301,10 @@ main(int argc,
 	    continue;
 	}
 
+#ifdef HAVE_ARES
+	ares_process(achannel, &readable, &writable);
+#endif
+
 	if (nfound == 0) {
 	    /* either we timed out or we were just
 	       polling for input.  Either way we want to continue
@@ -352,8 +315,6 @@ main(int argc,
 		bdump_send();
 	    else if (msgs_queued() || FD_ISSET(srv_socket, &readable))
 		handle_packet();
-	    else
-		syslog(LOG_ERR, "select weird?!?!");
 	}
     }
 }
@@ -435,6 +396,17 @@ do_net_setup(void)
     struct hostent *hp;
     char hostname[NS_MAXDNAME];
     int flags;
+#ifdef HAVE_ARES
+    int status;
+#endif
+
+#ifdef HAVE_ARES
+    status = ares_init(&achannel);
+    if (status != ARES_SUCCESS) {
+	syslog(LOG_ERR, "resolver init failed: %s", ares_strerror(status));
+	return 1;
+    }
+#endif
 
     if (gethostname(hostname, sizeof(hostname))) {
 	syslog(LOG_ERR, "no hostname: %m");
@@ -443,10 +415,10 @@ do_net_setup(void)
     hp = gethostbyname(hostname);
     if (!hp || hp->h_addrtype != AF_INET) {
 	syslog(LOG_ERR, "no gethostbyname repsonse");
-	strncpy(myname, hostname, sizeof(myname));
+	strncpy(myname, hostname, NS_MAXDNAME);
 	return 1;
     }
-    strncpy(myname, hp->h_name, sizeof(myname));
+    strncpy(myname, hp->h_name, NS_MAXDNAME);
     memcpy(&my_addr, hp->h_addr_list[0], hp->h_length);
 
     setservent(1);		/* keep file/connection open */
@@ -504,21 +476,6 @@ usage(void)
 	exit(2);
 }
 
-int
-packets_waiting(void)
-{
-    fd_set readable, initial;
-    struct timeval tv;
-
-    if (msgs_queued())
-	return 1;
-    FD_ZERO(&initial);
-    FD_SET(srv_socket, &initial);
-    readable = initial;
-    tv.tv_sec = tv.tv_usec = 0;
-    return (select(srv_socket + 1, &readable, NULL, NULL, &tv) > 0);
-}
-
 static RETSIGTYPE
 bye(int sig)
 {
@@ -688,6 +645,7 @@ do_reset(void)
     /* reset various things in the server's state */
     subscr_reset();
     server_reset();
+    realm_init();
     access_reinit();
     syslog(LOG_INFO, "restart completed");
     doreset = 0;
