@@ -3,9 +3,10 @@
 
 Unacked *rlm_nacklist = NULL;   /* not acked list for realm-realm
                                    packets */
-ZRealm *otherrealms;             /* points to an array of the known
+ZRealm **otherrealms = NULL;    /* points to an array of the known
                                    servers */
 int nrealms = 0;                /* number of other realms */
+int n_realm_slots = 0;          /* size of malloc'd otherrealms */
 
 /*
  * External Routines:
@@ -44,9 +45,6 @@ int nrealms = 0;                /* number of other realms */
  * void realm_deathgram()
  * tells other realms this server is going down
  *
- * void realm_wakeup()
- * tells other realms to resend their idea of their subs to us
- *
  * Code_t realm_control_dispatch(ZNotice_t *notice, int auth,
  *                               struct sockaddr_in *who, Server *server,
  *				 ZRealm *realm)
@@ -60,6 +58,7 @@ int nrealms = 0;                /* number of other realms */
  * do a database dump of foreign realm info
  *
  */
+static int realm_next_idx_by_idx(ZRealm *realm, int idx);
 static void realm_sendit(ZNotice_t *notice, struct sockaddr_in *who, int auth, ZRealm *realm, int ack_to_sender);
 #ifdef HAVE_KRB5
 static Code_t realm_sendit_auth(ZNotice_t *notice, struct sockaddr_in *who, int auth, ZRealm *realm, int ack_to_sender);
@@ -75,17 +74,205 @@ static Code_t ticket_retrieve(ZRealm *realm);
 static int ticket_lookup(char *realm);
 #endif
 
+static int
+is_usable(ZRealm_server *srvr)
+{
+    return !srvr->deleted && srvr->got_addr;
+}
+
+static int
+is_sendable(ZRealm_server *srvr)
+{
+    return !srvr->deleted && srvr->got_addr && !srvr->dontsend;
+}
+
+static void
+rlm_wakeup_cb(void *arg)
+{
+    ZRealm *realm = arg;
+    ZNotice_t snotice;
+    char *pack;
+    char rlm_recipient[REALM_SZ + 1];
+    int packlen, retval;
+
+    memset (&snotice, 0, sizeof (snotice));
+
+    snotice.z_opcode = REALM_BOOT;
+    snotice.z_port = srv_addr.sin_port;
+    snotice.z_class_inst = ZEPHYR_CTL_REALM;
+    snotice.z_class = ZEPHYR_CTL_CLASS;
+    snotice.z_recipient = "";
+    snotice.z_kind = ACKED;
+    snotice.z_num_other_fields = 0;
+    snotice.z_default_format = "";
+    snotice.z_sender = myname; /* my host name */
+    sprintf(rlm_recipient, "@%s", realm->name);
+    snotice.z_recipient = rlm_recipient;
+    snotice.z_default_format = "";
+    snotice.z_message = NULL;
+    snotice.z_message_len = 0;
+
+#ifdef HAVE_KRB5
+    if (!ticket_lookup(realm->name))
+	if ((retval = ticket_retrieve(realm)) != ZERR_NONE) {
+	    syslog(LOG_WARNING, "rlm_wakeup failed: %s",
+		   error_message(retval));
+	    return;
+	}
+#endif
+
+    if ((retval = ZFormatNotice(&snotice, &pack, &packlen, ZAUTH))
+	!= ZERR_NONE)
+    {
+	syslog(LOG_WARNING, "rlm_wakeup format: %s",
+	       error_message(retval));
+	return;
+    }
+    if ((retval = ZParseNotice(pack, packlen, &snotice))
+	!= ZERR_NONE) {
+	syslog(LOG_WARNING, "rlm_wakeup parse: %s",
+	       error_message(retval));
+	free(pack);
+	return;
+    }
+
+    realm_handoff(&snotice, 1, NULL, realm, 0);
+    free(pack);
+}
+
+static void
+rlm_set_server_address(ZRealm_server *srvr, struct hostent *hp)
+{
+    memmove(&srvr->addr.sin_addr, hp->h_addr, sizeof(struct in_addr));
+    /* use the server port */
+    srvr->addr.sin_port = srv_addr.sin_port;
+    srvr->addr.sin_family = AF_INET;
+    srvr->got_addr = 1;
+    if (is_sendable(srvr) && srvr->realm->state == REALM_NEW) {
+	srvr->realm->idx = realm_next_idx_by_idx(srvr->realm, srvr->realm->idx);
+	srvr->realm->state = REALM_TARDY;
+	/*
+	 * Set a timer to send a wakeup to this realm.  We do this rather
+	 * than just sending the notice now because, if we are not using
+	 * C-ARES, then we might be called during server startup before
+	 * the server is prepared to send notices.
+	 */
+	timer_set_rel(0, rlm_wakeup_cb, srvr->realm);
+    }
+}
+
+#ifdef HAVE_ARES
+
+static void rlm_server_address_timer_cb(void *srvr);
+static void rlm_server_address_lookup_cb(void *, int, int, struct hostent *);
+
+static void
+rlm_lookup_server_address(ZRealm_server *srvr)
+{
+    /* Cancel any pending future lookup. */
+    if (srvr->timer) {
+	timer_reset(srvr->timer);
+	srvr->timer = NULL;
+    }
+    ares_gethostbyname(achannel, srvr->name->string, AF_INET,
+		       rlm_server_address_lookup_cb, srvr);
+}
+
+static void
+rlm_server_address_timer_cb(void *arg)
+{
+    ZRealm_server *srvr = arg;
+
+    srvr->timer = NULL;
+    ares_gethostbyname(achannel, srvr->name->string, AF_INET,
+		       rlm_server_address_lookup_cb, arg);
+}
+
+static void
+rlm_server_address_lookup_cb(void *arg, int status, int timeouts,
+			     struct hostent *hp)
+{
+    ZRealm_server *srvr = arg;
+    int delay = 30;
+
+    if (status == ARES_SUCCESS) {
+	rlm_set_server_address(srvr, hp);
+	delay = 24 * 3600; /* Check again once per day */
+    } else {
+	syslog(LOG_WARNING, "%s: hostname lookup failed: %s",
+	       srvr->name->string, ares_strerror(status));
+    }
+
+    /*
+     * Set a timer to trigger another lookup.
+     * But, not if the server is deleted, which may have happened
+     * while we were waiting for ARES to finish the last lookup.
+     * Also, not if there is already a timer, which is possible if
+     * there were two outstanding lookups and we are the second to
+     * complete.  This can happen if we are asked to refresh the
+     * server list while a previous lookup is still in progress,
+     * since there is no convenient way to tell whether there is a
+     * lookup in progress.
+     */
+    if (!srvr->timer && !srvr->deleted)
+	srvr->timer = timer_set_rel(delay, rlm_server_address_timer_cb, arg);
+}
+
+#else
+
+static void
+rlm_lookup_server_address(ZRealm_server *srvr)
+{
+    struct hostent *hp;
+
+    hp = gethostbyname(srvr->name->string);
+    if (hp)
+	rlm_set_server_address(srvr, hp);
+    else
+	syslog(LOG_WARNING, "hostname failed, %s", srvr->name->string);
+}
+
+#endif
+
 static int
 realm_get_idx_by_addr(ZRealm *realm,
 		      struct sockaddr_in *who)
 {
-    struct sockaddr_in *addr;
+    ZRealm_server *srvr;
     int b;
 
     /* loop through the realms */
-    for (addr = realm->addrs, b = 0; b < realm->count; b++, addr++)
-	if (addr->sin_addr.s_addr == who->sin_addr.s_addr)
+    for (b = 0; b < realm->count; b++) {
+	srvr = realm->srvrs[b];
+	if (!is_usable(srvr))
+	    continue;
+	if (srvr->addr.sin_addr.s_addr == who->sin_addr.s_addr)
 	    return(b);
+    }
+
+    return 0;
+}
+
+static int
+realm_next_idx_by_idx(ZRealm *realm, int idx)
+{
+    ZRealm_server *srvr;
+    int b;
+
+    /* loop through the servers */
+    for (b = idx; b < realm->count; b++) {
+	srvr = realm->srvrs[b];
+	if (is_sendable(srvr))
+	    return(b);
+    }
+
+    /* recycle */
+    if (idx != 0)
+	for (b = 0; b < idx; b++) {
+	    srvr = realm->srvrs[b];
+	    if (is_sendable(srvr))
+		return(b);
+	}
 
     return 0;
 }
@@ -93,7 +280,6 @@ realm_get_idx_by_addr(ZRealm *realm,
 const char *
 realm_expand_realm(char *realmname)
 {
-    ZRealm *realm;
     int a;
 
     /* First, look for an exact match (case insensitive) */
@@ -102,9 +288,9 @@ realm_expand_realm(char *realmname)
 	return(ZGetRealm());
 #endif
 
-    for (realm = otherrealms, a = 0; a < nrealms; a++, realm++)
-      if (!strcasecmp(realm->name, realmname))
-	return(realm->name);
+    for (a = 0; a < nrealms; a++)
+      if (!strcasecmp(otherrealms[a]->name, realmname))
+	return(otherrealms[a]->name);
 
     /* No exact match. See if there's a partial match */
 #if defined(HAVE_KRB4) || defined(HAVE_KRB5)
@@ -112,21 +298,20 @@ realm_expand_realm(char *realmname)
 	return(ZGetRealm());
 #endif
 
-    for (realm = otherrealms, a = 0; a < nrealms; a++, realm++)
-	if (!strncasecmp(realm->name, realmname, strlen(realmname)))
-	    return(realm->name);
+    for (a = 0; a < nrealms; a++)
+	if (!strncasecmp(otherrealms[a]->name, realmname, strlen(realmname)))
+	    return(otherrealms[a]->name);
     return(realmname);
 }
 
 ZRealm *
 realm_get_realm_by_pid(int pid)
 {
-    ZRealm *realm;
     int a;
 
-    for (realm = otherrealms, a = 0; a < nrealms; a++, realm++)
-	if (realm->child_pid == pid)
-	    return(realm);
+    for (a = 0; a < nrealms; a++)
+	if (otherrealms[a]->child_pid == pid)
+	    return(otherrealms[a]);
 
     return 0;
 }
@@ -134,17 +319,16 @@ realm_get_realm_by_pid(int pid)
 void
 kill_realm_pids(void)
 {
-    ZRealm *realm;
     int a;
 
-    for (realm = otherrealms, a = 0; a < nrealms; a++, realm++)
-	if (realm->child_pid != 0)
-	    kill(realm->child_pid, 9);
+    for (a = 0; a < nrealms; a++)
+	if (otherrealms[a]->child_pid != 0)
+	    kill(otherrealms[a]->child_pid, 9);
 
     return;
 }
 
-ZRealmname *
+static ZRealmname *
 get_realm_lists(char *file)
 {
     ZRealmname *rlm_list, *rlm;
@@ -152,6 +336,7 @@ get_realm_lists(char *file)
     FILE *fp;
     char buf[REALM_SZ + NS_MAXDNAME + 1]; /* one for newline */
     char realm[REALM_SZ], server[NS_MAXDNAME + 1];
+    String *realm_name;
 
     nused = 0;
     if (!(fp = fopen(file, "r")))
@@ -170,25 +355,27 @@ get_realm_lists(char *file)
 	    syslog(LOG_CRIT, "bad format in %s", file);
 	    abort();
 	}
+	realm_name = make_string(realm, 0);
 	for (ii = 0; ii < nused; ii++) {
 	    /* look for this realm */
-	    if (!strcmp(rlm_list[ii].name, realm))
+	    if (rlm_list[ii].name == realm_name)
 		break;
 	}
 	if (ii < nused) {
+	    free_string(realm_name);
 	    rlm = &rlm_list[ii];
 	    if (rlm->nused +1 >= rlm->nservers) {
 		/* make more space */
-		rlm->servers = (char **)realloc((char *)rlm->servers,
-						(unsigned)rlm->nservers * 2 *
-						sizeof(char *));
+		rlm->servers = (struct _ZRealm_server *)
+		    realloc((char *)rlm->servers,
+			    (unsigned)rlm->nservers * 2 *
+			    sizeof(struct _ZRealm_server));
 		if (!rlm->servers) {
 		    syslog(LOG_CRIT, "get_realm_lists realloc");
 		    abort();
 		}
 		rlm->nservers *= 2;
 	    }
-	    rlm->servers[rlm->nused++] = strsave(server);
 	} else {
 	    /* new realm */
 	    if (nused + 1 >= ntotal) {
@@ -203,16 +390,24 @@ get_realm_lists(char *file)
 		ntotal *= 2;
 	    }
 	    rlm = &rlm_list[nused++];
-	    strcpy(rlm->name, realm);
+	    rlm->name = realm_name;
 	    rlm->nused = 0;
 	    rlm->nservers = 16;
-	    rlm->servers = (char **)malloc(rlm->nservers * sizeof(char *));
+	    rlm->servers = (struct _ZRealm_server *)
+		malloc(rlm->nservers * sizeof(struct _ZRealm_server));
 	    if (!rlm->servers) {
 		syslog(LOG_CRIT, "get_realm_lists malloc");
 		abort();
 	    }
-	    rlm->servers[rlm->nused++] = strsave(server);
 	}
+	memset(&rlm->servers[rlm->nused], 0, sizeof(struct _ZRealm_server));
+	if (*server == '/') {
+	    rlm->servers[rlm->nused].name = make_string(server + 1, 1);
+	    rlm->servers[rlm->nused].dontsend = 1;
+	} else {
+	    rlm->servers[rlm->nused].name = make_string(server, 1);
+	}
+	rlm->nused++;
     }
     if (nused + 1 >= ntotal) {
 	rlm_list = (ZRealmname *)realloc((char *)rlm_list,
@@ -223,7 +418,7 @@ get_realm_lists(char *file)
 	    abort();
 	}
     }
-    *rlm_list[nused].name = '\0';
+    rlm_list[nused].name = 0;
 
     fclose(fp);
     return(rlm_list);
@@ -234,7 +429,7 @@ realm_send_realms(void)
 {
     int cnt, retval;
     for (cnt = 0; cnt < nrealms; cnt++) {
-	retval = subscr_send_realm_subs(&otherrealms[cnt]);
+	retval = subscr_send_realm_subs(otherrealms[cnt]);
 	if (retval != ZERR_NONE)
 	    return(retval);
     }
@@ -277,34 +472,25 @@ realm_sender_in_realm(const char *realm, char *sender)
     return 0;
 }
 
-int sender_in_realm(ZNotice_t *notice)
-{
-  char *realm;
-
-  realm = strchr(notice->z_sender, '@');
-
-  if (!realm || !strcmp(realm + 1, ZGetRealm()))
-    return 1;
-
-  return 0;
-}
-
 ZRealm *
 realm_which_realm(struct sockaddr_in *who)
 {
-    ZRealm *realm;
-    struct sockaddr_in *addr;
+    ZRealm_server *srvr;
     int a, b;
 
     if (who->sin_port != srv_addr.sin_port)
 	return 0;
 
     /* loop through the realms */
-    for (realm = otherrealms, a = 0; a < nrealms; a++, realm++)
+    for (a = 0; a < nrealms; a++)
 	/* loop through the addresses for the realm */
-	for (addr = realm->addrs, b = 0; b < realm->count; b++, addr++)
-	    if (addr->sin_addr.s_addr == who->sin_addr.s_addr)
-		return(realm);
+	for (b = 0; b < otherrealms[a]->count; b++) {
+	    srvr = otherrealms[a]->srvrs[b];
+	    if (!is_usable(srvr))
+		continue;
+	    if (srvr->addr.sin_addr.s_addr == who->sin_addr.s_addr)
+		return(otherrealms[a]);
+	}
 
     return 0;
 }
@@ -313,17 +499,28 @@ ZRealm *
 realm_get_realm_by_name(char *name)
 {
     int a;
-    ZRealm *realm;
 
     /* First, look for an exact match (case insensitive) */
-    for (realm = otherrealms, a = 0; a < nrealms; a++, realm++)
-	if (!strcasecmp(realm->name, name))
-	    return(realm);
+    for (a = 0; a < nrealms; a++)
+	if (!strcasecmp(otherrealms[a]->name, name))
+	    return(otherrealms[a]);
 
     /* Failing that, look for an inexact match */
-    for (realm = otherrealms, a = 0; a < nrealms; a++, realm++)
-	if (!strncasecmp(realm->name, name, strlen(name)))
-	    return(realm);
+    for (a = 0; a < nrealms; a++)
+	if (!strncasecmp(otherrealms[a]->name, name, strlen(name)))
+	    return(otherrealms[a]);
+
+    return 0;
+}
+
+ZRealm *
+realm_get_realm_by_name_string(String *namestr)
+{
+    int a;
+
+    for (a = 0; a < nrealms; a++)
+	if (otherrealms[a]->namestr == namestr)
+	    return otherrealms[a];
 
     return 0;
 }
@@ -345,7 +542,7 @@ rlm_nack_cancel(register ZNotice_t *notice,
     }
 
     for (nacked = rlm_nacklist; nacked; nacked = nacked->next) {
-	if (&otherrealms[nacked->dest.rlm.rlm_idx] == which) {
+	if (nacked->dest.rlm.realm == which) {
 	    /* First, note the realm appears to be up */
 	    which->state = REALM_UP;
 	    if (ZCompareUID(&nacked->uid, &notice->z_uid)) {
@@ -468,65 +665,133 @@ realm_init(void)
     Client *client;
     ZRealmname *rlmnames;
     ZRealm *rlm;
-    int ii, jj, found;
-    struct in_addr *addresses;
-    struct hostent *hp;
-    char list_file[128];
+    int ii, jj, kk, nrlmnames, nsendable;
+    char realm_list_file[128];
     char rlmprinc[MAX_PRINCIPAL_SIZE];
 
-    sprintf(list_file, "%s/zephyr/%s", SYSCONFDIR, REALM_LIST_FILE);
-    rlmnames = get_realm_lists(list_file);
+    sprintf(realm_list_file, "%s/zephyr/%s", SYSCONFDIR, REALM_LIST_FILE);
+    rlmnames = get_realm_lists(realm_list_file);
     if (!rlmnames) {
 	zdbug((LOG_DEBUG, "No other realms"));
-	nrealms = 0;
+	/* should we nuke all existing server records? */
 	return;
     }
 
-    for (nrealms = 0; *rlmnames[nrealms].name; nrealms++);
+    for (nrlmnames = 0; rlmnames[nrlmnames].name; nrlmnames++);
 
-    otherrealms = (ZRealm *)malloc(nrealms * sizeof(ZRealm));
+    /*
+     * This happens only when we first start up.  Otherwise, otherrealms
+     * is grown as needed.
+     */
     if (!otherrealms) {
-	syslog(LOG_CRIT, "malloc failed in realm_init");
-	abort();
-    }
-
-    for (ii = 0; ii < nrealms; ii++) {
-	rlm = &otherrealms[ii];
-	strcpy(rlm->name, rlmnames[ii].name);
-
-	addresses = (struct in_addr *)malloc(rlmnames[ii].nused *
-					     sizeof(struct in_addr));
-	if (!addresses) {
+	otherrealms = (ZRealm **)malloc(nrlmnames * sizeof(ZRealm *));
+	if (!otherrealms) {
 	    syslog(LOG_CRIT, "malloc failed in realm_init");
 	    abort();
 	}
-	/* convert names to addresses */
-	found = 0;
-	for (jj = 0; jj < rlmnames[ii].nused; jj++) {
-	    hp = gethostbyname(rlmnames[ii].servers[jj]);
-	    if (hp) {
-		memmove(&addresses[found], hp->h_addr,
-			sizeof(struct in_addr));
-		found++;
-	    } else
-		syslog(LOG_WARNING, "hostname failed, %s",
-		       rlmnames[ii].servers[jj]);
-	    /* free the hostname */
-	    free(rlmnames[ii].servers[jj]);
+	memset(otherrealms, 0, (nrlmnames * sizeof(ZRealm *)));
+	n_realm_slots = nrlmnames;
+    }
+
+    /* ii: entry in rlmnames */
+    for (ii = 0; ii < nrlmnames; ii++) {
+	nsendable = 0;
+	rlm = realm_get_realm_by_name_string(rlmnames[ii].name);
+	if (rlm) {
+	    /* jj: server entry in otherrealms */
+	    /* kk: server entry in rlmnames */
+	    for (jj = 0; jj < rlm->count; jj++) {
+		rlm->srvrs[jj]->deleted = 1;
+		for (kk = 0; kk < rlmnames[ii].nused; kk++) {
+		    if (rlmnames[ii].servers[kk].name != rlm->srvrs[jj]->name)
+			continue;
+		    /* update existing server */
+		    rlm->srvrs[jj]->dontsend = rlmnames[ii].servers[kk].dontsend;
+		    rlm->srvrs[jj]->deleted = 0;
+		    rlm_lookup_server_address(rlm->srvrs[jj]);
+		    if (is_sendable(rlm->srvrs[jj])) nsendable++;
+
+		    /* mark realm.list server entry used */
+		    rlmnames[ii].servers[kk].deleted = 1;
+		    break;
+		}
+		if (rlm->srvrs[jj]->deleted && rlm->srvrs[jj]->timer) {
+		    timer_reset(rlm->srvrs[jj]->timer);
+		    rlm->srvrs[jj]->timer = NULL;
+		}
+	    }
+	    for (jj = kk = 0; kk < rlmnames[ii].nused; kk++)
+		if (!rlmnames[ii].servers[kk].deleted) jj++;
+
+	    rlm->srvrs = realloc(rlm->srvrs,
+				 (rlm->count + jj) * sizeof(ZRealm_server *));
+	    if (!rlm->srvrs) {
+		syslog(LOG_CRIT, "realloc failed in realm_init");
+		abort();
+	    }
+	    for (kk = 0; kk < rlmnames[ii].nused; kk++) {
+		if (rlmnames[ii].servers[kk].deleted) continue;
+		rlm->srvrs[rlm->count] = malloc(sizeof(ZRealm_server));
+		if (!rlm->srvrs[rlm->count]) {
+		    syslog(LOG_CRIT, "realloc failed in realm_init");
+		    abort();
+		}
+		*(rlm->srvrs[rlm->count]) = rlmnames[ii].servers[kk];
+		rlm->srvrs[rlm->count]->realm = rlm;
+		rlm_lookup_server_address(rlm->srvrs[rlm->count]);
+		if (is_sendable(rlm->srvrs[rlm->count])) nsendable++;
+		rlm->count++;
+	    }
+	    /* The current server might have been deleted or marked dontsend.
+	       Advance to one we can use, if necessary. */
+	    if (nsendable) {
+		rlm->idx = realm_next_idx_by_idx(rlm, rlm->idx);
+	    } else {
+		rlm->idx = 0;
+		rlm->state = REALM_NEW;
+	    }
+	    free(rlmnames[ii].servers);
+	    continue;
 	}
-	rlm->count = found;
-	rlm->addrs = (struct sockaddr_in *)malloc(found *
-						  sizeof (struct sockaddr_in));
-	if (!rlm->addrs) {
+
+	if (nrealms >= n_realm_slots) {
+	    otherrealms = realloc(otherrealms,
+				  n_realm_slots * 2 * sizeof(ZRealm *));
+	    if (!otherrealms) {
+		syslog(LOG_CRIT, "realloc failed in realm_init");
+		abort();
+	    }
+	    memset(otherrealms + n_realm_slots, 0,
+		   n_realm_slots * sizeof(ZRealm *));
+	    n_realm_slots *= 2;
+	}
+
+	rlm = (ZRealm *) malloc(sizeof(ZRealm));
+	if (!rlm) {
+	    syslog(LOG_CRIT, "malloc failed in realm_init");
+	    abort();
+	}
+	memset(rlm, 0, sizeof(ZRealm));
+	otherrealms[nrealms++] = rlm;
+
+	rlm->namestr = rlmnames[ii].name;
+	rlm->name = rlm->namestr->string;
+	rlm->state = REALM_NEW;
+
+	/* convert names to addresses */
+	rlm->count = rlmnames[ii].nused;
+	rlm->srvrs = malloc(rlm->count * sizeof(ZRealm_server *));
+	if (!rlm->srvrs) {
 	    syslog(LOG_CRIT, "malloc failed in realm_init");
 	    abort();
 	}
 	for (jj = 0; jj < rlm->count; jj++) {
-	    rlm->addrs[jj].sin_family = AF_INET;
-	    /* use the server port */
-	    rlm->addrs[jj].sin_port = srv_addr.sin_port;
-	    rlm->addrs[jj].sin_addr = addresses[jj];
+	    rlm->srvrs[jj] = &rlmnames[ii].servers[jj];
+	    rlm->srvrs[jj]->realm = rlm;
+	    rlm_lookup_server_address(rlm->srvrs[jj]);
+	    if (is_sendable(rlm->srvrs[jj])) nsendable++;
 	}
+
 	client = (Client *) malloc(sizeof(Client));
 	if (!client) {
 	    syslog(LOG_CRIT, "malloc failed in realm_init");
@@ -552,15 +817,12 @@ realm_init(void)
 	client->addr.sin_addr.s_addr = 0;
 
 	rlm->client = client;
-	rlm->idx = (rlm->count) ? random() % rlm->count : 0;
+	rlm->idx = (nsendable) ?
+	    realm_next_idx_by_idx(rlm, (random() % rlm->count)) : 0;
 	rlm->subs = NULL;
 	rlm->remsubs = NULL;
 	rlm->child_pid = 0;
-	/* Assume the best */
-	rlm->state = REALM_TARDY;
 	rlm->have_tkt = 1;
-	free(rlmnames[ii].servers);
-	free(addresses);
     }
     free(rlmnames);
 }
@@ -572,12 +834,13 @@ realm_deathgram(Server *server)
     int jj = 0;
 
     /* Get it out once, and assume foreign servers will share */
-    for (realm = otherrealms, jj = 0; jj < nrealms; jj++, realm++) {
+    for (jj = 0; jj < nrealms; jj++) {
 	ZNotice_t snotice;
 	char *pack;
 	char rlm_recipient[REALM_SZ + 1];
 	int packlen, retval;
 
+	realm = otherrealms[jj];
 	memset (&snotice, 0, sizeof (snotice));
 
 	snotice.z_kind = ACKED;
@@ -625,73 +888,6 @@ realm_deathgram(Server *server)
     }
 }
 
-void
-realm_wakeup(void)
-{
-    int jj, found = 0;
-    ZRealm *realm;
-
-    for (jj = 1; jj < nservers; jj++) {    /* skip limbo server */
-	if (jj != me_server_idx && otherservers[jj].state == SERV_UP)
-	    found++;
-    }
-
-    if (nservers < 2 || !found) {
-	/* if we're the only server up, send a REALM_BOOT to one of their
-	   servers here */
-	for (realm = otherrealms, jj = 0; jj < nrealms; jj++, realm++) {
-	    ZNotice_t snotice;
-	    char *pack;
-	    char rlm_recipient[REALM_SZ + 1];
-	    int packlen, retval;
-
-	    memset (&snotice, 0, sizeof (snotice));
-
-	    snotice.z_opcode = REALM_BOOT;
-	    snotice.z_port = srv_addr.sin_port;
-	    snotice.z_class_inst = ZEPHYR_CTL_REALM;
-	    snotice.z_class = ZEPHYR_CTL_CLASS;
-	    snotice.z_recipient = "";
-	    snotice.z_kind = ACKED;
-	    snotice.z_num_other_fields = 0;
-	    snotice.z_default_format = "";
-	    snotice.z_sender = myname; /* my host name */
-	    sprintf(rlm_recipient, "@%s", realm->name);
-	    snotice.z_recipient = rlm_recipient;
-	    snotice.z_default_format = "";
-	    snotice.z_message = NULL;
-	    snotice.z_message_len = 0;
-
-#ifdef HAVE_KRB5
-	    if (!ticket_lookup(realm->name))
-		if ((retval = ticket_retrieve(realm)) != ZERR_NONE) {
-		    syslog(LOG_WARNING, "rlm_wakeup failed: %s",
-			   error_message(retval));
-		    continue;
-		}
-#endif
-
-	    if ((retval = ZFormatNotice(&snotice, &pack, &packlen, ZAUTH))
-		!= ZERR_NONE)
-	    {
-		syslog(LOG_WARNING, "rlm_wakeup format: %s",
-		       error_message(retval));
-		return;
-	    }
-	    if ((retval = ZParseNotice(pack, packlen, &snotice))
-		!= ZERR_NONE) {
-		syslog(LOG_WARNING, "rlm_wakeup parse: %s",
-		       error_message(retval));
-		free(pack);
-		return;
-	    }
-
-	    realm_handoff(&snotice, 1, NULL, realm, 0);
-	    free(pack);
-	}
-    }
-}
-
 static Code_t
 realm_ulocate_dispatch(ZNotice_t *notice,
 		       int auth,
@@ -770,8 +966,9 @@ realm_control_dispatch(ZNotice_t *notice,
 	    ack(notice, who);
 	}
     } else if (!strcmp(opcode, REALM_BOOT)) {
-	zdbug((LOG_DEBUG, "got a REALM_BOOT from %d (me %d)", server, me_server));
-	realm->state = REALM_STARTING;
+	zdbug((LOG_DEBUG, "got a REALM_BOOT from %s",
+               inet_ntoa(server->addr.sin_addr)));
+	if (realm->state != REALM_UP) realm->state = REALM_STARTING;
 	realm_set_server(who, realm);
 #ifdef REALM_MGMT
 	/* resend subscriptions but only if this was to us */
@@ -827,13 +1024,17 @@ realm_new_server(struct sockaddr_in *sin,
 	    /* Not exactly */
 	    if (!rlm || (rlm != realm))
 		return ZSRV_NORLM;
-	    realm->idx = realm_get_idx_by_addr(realm, &sinaddr);
+	    /* Validate the hint */
+	    realm->idx =
+		realm_next_idx_by_idx(realm, realm_get_idx_by_addr(realm,
+								   &sinaddr));
 	} else {
-	    realm->idx = (realm->idx + 1) % realm->count;
+	    realm->idx = realm_next_idx_by_idx(realm, (realm->idx + 1) %
+					       realm->count);
 	}
-	zdbug((LOG_DEBUG, "rlm_new_srv: switched servers (%s)", inet_ntoa((realm->addrs[realm->idx]).sin_addr)));
+	zdbug((LOG_DEBUG, "rlm_new_srv: switched servers (%s)", inet_ntoa((realm->srvrs[realm->idx]->addr).sin_addr)));
     } else {
-      zdbug((LOG_DEBUG, "rlm_new_srv: not switching servers (%s)", inet_ntoa((realm->addrs[realm->idx]).sin_addr)));
+      zdbug((LOG_DEBUG, "rlm_new_srv: not switching servers (%s)", inet_ntoa((realm->srvrs[realm->idx]->addr).sin_addr)));
     }
     return 0;
 }
@@ -843,13 +1044,21 @@ realm_set_server(struct sockaddr_in *sin,
 		 ZRealm *realm)
 {
     ZRealm *rlm;
+    int idx;
 
     rlm = realm_which_realm(sin);
     /* Not exactly */
     if (!rlm || (rlm != realm))
 	return ZSRV_NORLM;
-    realm->idx = realm_get_idx_by_addr(realm, sin);
-    zdbug((LOG_DEBUG, "rlm_pick_srv: switched servers (%s)", inet_ntoa((realm->addrs[realm->idx]).sin_addr)));
+    idx = realm_get_idx_by_addr(realm, sin);
+
+    /* Not exactly */
+    if (!is_sendable(realm->srvrs[idx]))
+	return ZSRV_NORLM;
+
+    realm->idx = idx;
+
+    zdbug((LOG_DEBUG, "rlm_pick_srv: switched servers (%s)", inet_ntoa((realm->srvrs[realm->idx]->addr).sin_addr)));
 
     return 0;
 }
@@ -899,7 +1108,16 @@ realm_sendit(ZNotice_t *notice,
     Code_t retval;
     Unacked *nacked;
 
+    if (realm->count == 0 || realm->state == REALM_NEW) {
+	/* XXX we should have a queue or something */
+	syslog(LOG_WARNING, "rlm_sendit no servers for %s", realm->name);
+	return;
+    }
+
     notice->z_auth = auth;
+    notice->z_authent_len = 0;
+    notice->z_ascii_authent = "";
+    notice->z_checksum = 0;
 
     /* format the notice */
     if ((retval = ZFormatRawNotice(notice, &pack, &packlen)) != ZERR_NONE) {
@@ -909,7 +1127,7 @@ realm_sendit(ZNotice_t *notice,
     }
 
     /* now send */
-    if ((retval = ZSetDestAddr(&realm->addrs[realm->idx])) != ZERR_NONE) {
+    if ((retval = ZSetDestAddr(&realm->srvrs[realm->idx]->addr)) != ZERR_NONE) {
 	syslog(LOG_WARNING, "rlm_sendit set addr: %s",
 	       error_message(retval));
 	free(pack);
@@ -932,7 +1150,7 @@ realm_sendit(ZNotice_t *notice,
 
     memset(nacked, 0, sizeof(Unacked));
     nacked->packet = pack;
-    nacked->dest.rlm.rlm_idx = realm - otherrealms;
+    nacked->dest.rlm.realm = realm;
     nacked->dest.rlm.rlm_srv_idx = realm->idx;
     nacked->packsz = packlen;
     nacked->uid = notice->z_uid;
@@ -971,12 +1189,12 @@ rlm_rexmit(void *arg)
 
     zdbug((LOG_DEBUG,"rlm_rexmit"));
 
-    realm = &otherrealms[nackpacket->dest.rlm.rlm_idx];
+    realm = nackpacket->dest.rlm.realm;
 
     zdbug((LOG_DEBUG, "rlm_rexmit: sending to %s:%d (%d)",
 	   realm->name, realm->idx, nackpacket->rexmits));
 
-    if (realm->count == 0)
+    if (realm->count == 0 || realm->state == REALM_NEW)
 	return;
 
     /* Check to see if we've retransmitted as many times as we can */
@@ -998,17 +1216,18 @@ rlm_rexmit(void *arg)
 	(nackpacket->rexmits &&
 	 !((nackpacket->rexmits+1) % (NUM_REXMIT_TIMES/3))))
     {
-	realm->idx = (realm->idx + 1) % realm->count;
+	realm->idx = realm_next_idx_by_idx(realm, (realm->idx + 1) %
+					   realm->count);
 	zdbug((LOG_DEBUG, "rlm_rexmit: %s switching servers:%d (%s)",
 	       realm->name, realm->idx,
-	       inet_ntoa((realm->addrs[realm->idx]).sin_addr)));
+	       inet_ntoa((realm->srvrs[realm->idx]->addr).sin_addr)));
     }
 
     /* throttle back if it looks like the realm is down */
     if ((realm->state != REALM_DEAD) ||
 	((nackpacket->rexmits % (realm->count+1)) == 1)) {
 	/* do the retransmit */
-	retval = ZSetDestAddr(&realm->addrs[realm->idx]);
+	retval = ZSetDestAddr(&realm->srvrs[realm->idx]->addr);
 	if (retval != ZERR_NONE) {
 	    syslog(LOG_WARNING, "rlm_rexmit set addr: %s",
 		   error_message(retval));
@@ -1021,10 +1240,10 @@ rlm_rexmit(void *arg)
 	/* no per-server nack queues for foreign realms yet, doesn't matter */
 	nackpacket->dest.rlm.rlm_srv_idx = realm->idx;
 	zdbug((LOG_DEBUG, "rlm_rexmit(%s): send to %s", realm->name,
-	       inet_ntoa((realm->addrs[realm->idx]).sin_addr)));
+	       inet_ntoa((realm->srvrs[realm->idx]->addr).sin_addr)));
     } else {
 	zdbug((LOG_DEBUG, "rlm_rexmit(%s): not sending to %s", realm->name,
-	       inet_ntoa((realm->addrs[realm->idx]).sin_addr)));
+	       inet_ntoa((realm->srvrs[realm->idx]->addr).sin_addr)));
     }
 
     /* reset the timer */
@@ -1032,9 +1251,10 @@ rlm_rexmit(void *arg)
     nackpacket->timer =
 	timer_set_rel(rexmit_times[nackpacket->rexmits%NUM_REXMIT_TIMES],
 		      rlm_rexmit, nackpacket);
-    if (rexmit_times[nackpacket->rexmits%NUM_REXMIT_TIMES] == -1)
+    if (rexmit_times[nackpacket->rexmits%NUM_REXMIT_TIMES] == -1) {
 	zdbug((LOG_DEBUG, "rlm_rexmit(%s): would send at -1 to %s",
-	       realm->name, inet_ntoa((realm->addrs[realm->idx]).sin_addr)));
+	       realm->name, inet_ntoa((realm->srvrs[realm->idx]->addr).sin_addr)));
+    }
 
     return;
 }
@@ -1045,13 +1265,16 @@ realm_dump_realms(FILE *fp)
     register int ii, jj;
 
     for (ii = 0; ii < nrealms; ii++) {
-	(void) fprintf(fp, "%d:%s\n", ii, otherrealms[ii].name);
-	for (jj = 0; jj < otherrealms[ii].count; jj++) {
-	    (void) fprintf(fp, "\t%s\n",
-			   inet_ntoa(otherrealms[ii].addrs[jj].sin_addr));
+	(void) fprintf(fp, "%d:%s\n", ii, otherrealms[ii]->name);
+	for (jj = 0; jj < otherrealms[ii]->count; jj++) {
+	    (void) fprintf(fp, "\t%s%s%s%s\n",
+			   inet_ntoa(otherrealms[ii]->srvrs[jj]->addr.sin_addr),
+			   otherrealms[ii]->srvrs[jj]->dontsend ? " nosend" : "",
+			   otherrealms[ii]->srvrs[jj]->got_addr ? " gotaddr" : "",
+			   otherrealms[ii]->srvrs[jj]->deleted ? " deleted" : "");
 	}
 	/* dump the subs */
-	subscr_dump_subs(fp, otherrealms[ii].subs);
+	subscr_dump_subs(fp, otherrealms[ii]->subs);
     }
 }
 
@@ -1070,7 +1293,7 @@ realm_auth_sendit_nacked(char *buffer, int packlen, ZRealm *realm,
 
     memset(nacked, 0, sizeof(Unacked));
     nacked->packet = buffer;
-    nacked->dest.rlm.rlm_idx = realm - otherrealms;
+    nacked->dest.rlm.realm = realm;
     nacked->dest.rlm.rlm_srv_idx = realm->idx;
     nacked->packsz = packlen;
     nacked->uid = uid;
@@ -1104,6 +1327,12 @@ realm_sendit_auth(ZNotice_t *notice,
     char multi[64];
     ZNotice_t partnotice, newnotice;
 
+    if (realm->count == 0 || realm->state == REALM_NEW) {
+	/* XXX we should have a queue or something */
+	syslog(LOG_WARNING, "rlm_sendit_auth no servers for %s", realm->name);
+	return ZERR_INTERNAL;
+    }
+
     offset = 0;
 
     buffer = (char *)malloc(sizeof(ZPacket_t));
@@ -1123,7 +1352,7 @@ realm_sendit_auth(ZNotice_t *notice,
 	       error_message(retval));
 
     if (!retval) {
-	retval = ZSetDestAddr(&realm->addrs[realm->idx]);
+	retval = ZSetDestAddr(&realm->srvrs[realm->idx]->addr);
 	if (retval)
 	    syslog(LOG_WARNING, "rlm_sendit_auth: ZSetDestAddr: %s",
 		   error_message(retval));
@@ -1139,7 +1368,7 @@ realm_sendit_auth(ZNotice_t *notice,
      */
 
     if (!retval &&
-	((notice->z_message_len+hdrlen > sizeof(ZPacket_t)) ||
+	((notice->z_message_len+hdrlen > (int)sizeof(ZPacket_t)) ||
 	 (notice->z_message_len+hdrlen > Z_MAXPKTLEN))) {
 
 	/* Reallocate buffers inside the refragmenter */
@@ -1261,13 +1490,14 @@ ticket_lookup(char *realm)
     krb5_error_code result;
     krb5_timestamp sec;
     krb5_ccache ccache;
-    krb5_creds creds_in, *creds;
+    krb5_creds creds_in, creds;
 
     result = krb5_cc_default(Z_krb5_ctx, &ccache);
     if (result)
       return 0;
 
     memset(&creds_in, 0, sizeof(creds_in));
+    memset(&creds, 0, sizeof(creds));
 
     result = krb5_cc_get_principal(Z_krb5_ctx, ccache, &creds_in.client);
     if (result) {
@@ -1285,18 +1515,18 @@ ticket_lookup(char *realm)
       return 0;
     }
 
-    result = krb5_get_credentials(Z_krb5_ctx, 0 /* flags */, ccache,
-                                  &creds_in, &creds);
+    result = krb5_cc_retrieve_cred(Z_krb5_ctx, ccache, 0, &creds_in, &creds);
     krb5_cc_close(Z_krb5_ctx, ccache);
     /* good ticket? */
 
     krb5_timeofday (Z_krb5_ctx, &sec);
     krb5_free_cred_contents(Z_krb5_ctx, &creds_in); /* hope this is OK */
-    if ((result == 0) && (sec < creds->times.endtime)) {
-      krb5_free_creds(Z_krb5_ctx, creds);
+    if ((result == 0) && (sec < creds.times.endtime)) {
+      krb5_free_cred_contents(Z_krb5_ctx, &creds);
       return (1);
     }
-    if (!result) krb5_free_creds(Z_krb5_ctx, creds);
+    if (!result)
+      krb5_free_cred_contents(Z_krb5_ctx, &creds);
 
     return (0);
 }
