@@ -16,10 +16,14 @@
 
 #ifndef lint
 #ifndef SABER
-static const char rcsid_subscr_c[] = "$Id$";
+static const char rcsid_subscr_c[] = "$Id: b788663061783fb37d46ee7bc8d034cc288152b7 $";
 #endif
 #endif
 
+#ifndef INADDR_NONE
+#define INADDR_NONE 0xffffffff
+#endif
+
 /*
  * The subscription manager.
  *
@@ -66,22 +70,6 @@ C_Block	serv_key;
 Sched	serv_ksched;
 #endif
 
-/* for compatibility when sending subscription information to old clients */
-
-#ifdef OLD_COMPAT
-#define	OLD_ZEPHYR_VERSION	"ZEPH0.0"
-#define	OLD_CLIENT_INCOMPSUBS	"INCOMP"
-static void old_compat_subscr_sendlist(ZNotice_t *notice, int auth,
-				       struct sockaddr_in *who);
-extern int old_compat_count_subscr;	/* counter of old use */
-#endif /* OLD_COMPAT */
-#ifdef NEW_COMPAT
-#define NEW_OLD_ZEPHYR_VERSION	"ZEPH0.1"
-static void new_old_compat_subscr_sendlist(ZNotice_t *notice, int auth,
-					   struct sockaddr_in *who);
-extern int new_compat_count_subscr;	/* counter of old use */
-#endif /* NEW_COMPAT */
-
 static Code_t add_subscriptions(Client *who, Destlist *subs_queue,
 				ZNotice_t *notice, Server *server);
 static Destlist *extract_subscriptions(ZNotice_t *notice);
@@ -158,14 +146,14 @@ add_subscriptions(Client *who,
 	    }
 	    acl = class_get_acl(subs->dest.classname);
 	    if (acl && !realm) {
-		if (!access_check(sender->string, acl, SUBSCRIBE)) {
+		if (!access_check(sender->string, &who->addr, acl, SUBSCRIBE)) {
 		    syslog(LOG_WARNING, "subscr unauth %s class %s",
 			   sender->string, subs->dest.classname->string);
 		    free_subscription(subs); /* free this one - denied */
 		    continue; /* the for loop */
 		}
 		if (wildcard_instance == subs->dest.inst) {
-		    if (!access_check(sender->string, acl, INSTWILD)) {
+		    if (!access_check(sender->string, &who->addr, acl, INSTWILD)) {
 			syslog(LOG_WARNING,
 			       "subscr unauth %s class %s wild inst",
 			       sender->string, subs->dest.classname->string);
@@ -176,13 +164,12 @@ add_subscriptions(Client *who,
 	    }
 	}
 	if (realm && !bdumping) {
-	        retval = subscr_realm_sendit(who, subs, notice, realm);
-	        if (retval != ZERR_NONE) {
-		    free_subscription(subs);
-		    continue; /* the for loop */
+	    retval = subscr_realm_sendit(who, subs, notice, realm);
+	    if (retval != ZERR_NONE) {
+		free_subscription(subs);
+		continue; /* the for loop */
 	    } else {
-	            /* Indicates we leaked traffic back to our realm */
-		    free_subscription(subs); /* free this one, wil get from
+		free_subscription(subs); /* free this one, wil get from
 						ADD */
 	    }
 	} else {
@@ -364,7 +351,6 @@ subscr_realm_cancel(struct sockaddr_in *sin,
 		    ZRealm *realm)
 {
     Destlist *cancel_subs, *subs, *client_subs, *next, *next2;
-    Code_t retval;
     int found = 0;
 
     if (!realm)
@@ -383,7 +369,7 @@ subscr_realm_cancel(struct sockaddr_in *sin,
             next2 = client_subs->next;
             if (ZDest_eq(&client_subs->dest, &subs->dest)) {
                 Destlist_delete(client_subs);
-                retval = triplet_deregister(realm->client, &client_subs->dest, realm);
+                triplet_deregister(realm->client, &client_subs->dest, realm);
 		free_subscription(client_subs);
                 found = 1;
                 break;
@@ -444,22 +430,6 @@ subscr_sendlist(ZNotice_t *notice,
     struct sockaddr_in send_to_who;
     Code_t retval;
 
-#ifdef OLD_COMPAT
-    if (strcmp(notice->z_version, OLD_ZEPHYR_VERSION) == 0) {
-	/* we are talking to an old client; use the old-style
-	   acknowledgement-message */
-	old_compat_subscr_sendlist(notice, auth, who);
-	return;
-    }
-#endif /* OLD_COMPAT */
-#ifdef NEW_COMPAT
-    if (strcmp(notice->z_version, NEW_OLD_ZEPHYR_VERSION) == 0) {
-	/* we are talking to a new old client; use the new-old-style
-	   acknowledgement-message */
-	new_old_compat_subscr_sendlist(notice, auth, who);
-	return;
-    }
-#endif /* NEW_COMPAT */
     answer = subscr_marshal_subs(notice, auth, who, &found);
     send_to_who = *who;
     send_to_who.sin_port = notice->z_port;  /* Return port */
@@ -588,179 +558,6 @@ subscr_marshal_subs(ZNotice_t *notice,
     return answer;
 }
 
-#ifdef NEW_COMPAT
-static void
-new_old_compat_subscr_sendlist(notice, auth, who)
-    ZNotice_t *notice;
-    int auth;
-    struct sockaddr_in *who;
-{
-    Code_t retval;
-    ZNotice_t reply;
-    ZPacket_t reppacket;
-    int packlen, found, count, initfound, zerofound;
-    char buf[64];
-    const char **answer;
-    struct sockaddr_in send_to_who;
-    int i;
-
-    new_compat_count_subscr++;
-
-    syslog(LOG_INFO, "new old subscr, %s", inet_ntoa(who->sin_addr));
-    reply = *notice;
-    reply.z_kind = SERVACK;
-    reply.z_authent_len = 0; /* save some space */
-    reply.z_auth = 0;
-
-    send_to_who = *who;
-    send_to_who.sin_port = notice->z_port;  /* Return port */
-
-    retval = ZSetDestAddr(&send_to_who);
-    if (retval != ZERR_NONE) {
-	syslog(LOG_WARNING, "new_old_subscr_sendlist set addr: %s",
-	       error_message(retval));
-	return;
-    }
-
-    /* retrieve  the subscriptions */
-    answer = subscr_marshal_subs(notice, auth, who, &found);
-
-    /* note that when there are no subscriptions, found == 0, so
-       we needn't worry about answer being NULL since
-       ZFormatSmallRawNoticeList won't reference the pointer */
-
-    /* send 5 at a time until we are finished */
-    count = found?((found-1) / 5 + 1):1;	/* total # to be sent */
-    i = 0;					/* pkt # counter */
-    initfound = found;
-    zerofound = (found == 0);
-    while (found > 0 || zerofound) {
-	packlen = sizeof(reppacket);
-	sprintf(buf, "%d/%d", ++i, count);
-	reply.z_opcode = buf;
-	retval = ZFormatSmallRawNoticeList(&reply,
-					   answer + (initfound - found)
-					    * NUM_FIELDS,
-					   ((found > 5) ? 5 : found)
-					    * NUM_FIELDS,
-					   reppacket, &packlen);
-	if (retval != ZERR_NONE) {
-	    syslog(LOG_ERR, "subscr_sendlist format: %s",
-		   error_message(retval));
-	    if (answer)
-		free(answer);
-	    return;
-	}
-	retval = ZSendPacket(reppacket, packlen, 0);
-	if (retval != ZERR_NONE) {
-	    syslog(LOG_WARNING, "subscr_sendlist xmit: %s",
-		   error_message(retval));
-	    if (answer)
-		free(answer);
-	    return;
-	}
-	found -= 5;
-	zerofound = 0;
-    }
-    if (answer)
-	free(answer);
-}
-#endif /* NEW_COMPAT */
-
-#ifdef OLD_COMPAT
-static void
-old_compat_subscr_sendlist(notice, auth, who)
-    ZNotice_t *notice;
-    int auth;
-    struct sockaddr_in *who;
-{
-    Client *client = client_find(&who->sin_addr, notice->z_port);
-    Destlist *subs;
-    Code_t retval;
-    ZNotice_t reply;
-    ZPacket_t reppacket;
-    int packlen, i, found = 0;
-    char **answer = NULL;
-
-    old_compat_count_subscr++;
-
-    syslog(LOG_INFO, "old old subscr, %s", inet_ntoa(who->sin_addr));
-    if (client && client->subs) {
-
-	/* check authenticity here.  The user must be authentic to get
-	   a list of subscriptions. If he is not subscribed to
-	   anything, the above test fails, and he gets a response
-	   indicating no subscriptions */
-
-	if (!auth) {
-	    clt_ack(notice, who, AUTH_FAILED);
-	    return;
-	}
-
-	for (subs = client->subs; subs; subs = subs->next)
-	    found++;
-	/* found is now the number of subscriptions */
-
-	/* coalesce the subscription information into a list of char *'s */
-	answer = (char **) malloc(found * NUM_FIELDS * sizeof(char *));
-	if (!answer) {
-	    syslog(LOG_ERR, "old_subscr_sendlist no mem(answer)");
-	    found = 0;
-	} else {
-	    i = 0;
-	    for (subs = client->subs; subs; subs = subs->next) {
-		answer[i*NUM_FIELDS] = subs->dest.classname->string;
-		answer[i*NUM_FIELDS + 1] = subs->dest.inst->string;
-		answer[i*NUM_FIELDS + 2] = subs->dest.recip->string;
-		i++;
-	    }
-	}
-    }
-
-    /* note that when there are no subscriptions, found == 0, so
-       we needn't worry about answer being NULL */
-
-    reply = *notice;
-    reply.z_kind = SERVACK;
-    reply.z_authent_len = 0; /* save some space */
-    reply.z_auth = 0;
-
-    /* if it's too long, chop off one at a time till it fits */
-    while ((retval = ZFormatSmallRawNoticeList(&reply, answer,
-					       found * NUM_FIELDS,
-					       reppacket,
-					       &packlen)) != ZERR_PKTLEN) {
-	found--;
-	reply.z_opcode = OLD_CLIENT_INCOMPSUBS;
-    }
-    if (retval != ZERR_NONE) {
-	syslog(LOG_ERR, "old_subscr_sendlist format: %s",
-	       error_message(retval));
-	if (answer)
-	    free(answer);
-	return;
-    }
-    retval = ZSetDestAddr(who);
-    if (retval != ZERR_NONE) {
-	syslog(LOG_WARNING, "subscr_sendlist set addr: %s",
-	       error_message(retval));
-	if (answer)
-	    free(answer);
-	return;
-    }
-    retval = ZSendPacket(reppacket, packlen, 0);
-    if (retval != ZERR_NONE) {
-	syslog(LOG_WARNING, "subscr_sendlist xmit: %s",
-	       error_message(retval));
-	if (answer)
-	    free(answer);
-	return;
-    }
-    if (answer)
-	free(answer);
-}
-#endif /* OLD_COMPAT */
-
 static Code_t
 subscr_send_some_subs(char *opcode, struct sockaddr_in *addr, Destlist *subs, char *subject)
 {
@@ -1162,6 +959,7 @@ subscr_unsub_sendit(Client *who,
   list[1] = subs->dest.inst->string;
   list[2] = "";
 
+  memset(&unotice, 0, sizeof(unotice));
   unotice.z_class = ZEPHYR_CTL_CLASS;
   unotice.z_class_inst = ZEPHYR_CTL_REALM;
   unotice.z_opcode = REALM_UNSUBSCRIBE;
@@ -1255,6 +1053,7 @@ subscr_realm_subs(ZRealm *realm)
     text[4] = subs->dest.recip->string;
 
     /* format snotice */
+    memset (&snotice, 0, sizeof(snotice));
     snotice.z_class_inst = ZEPHYR_CTL_REALM;
     snotice.z_opcode = REALM_REQ_SUBSCRIBE;
     snotice.z_port = 0;
@@ -1360,13 +1159,13 @@ subscr_check_foreign_subs(ZNotice_t *notice,
 		    return ZSRV_CLASSRESTRICTED;
 		}
 	    }
-	    if (!access_check(sender->string, acl, SUBSCRIBE)) {
+	    if (!access_check(sender->string, who, acl, SUBSCRIBE)) {
 		syslog(LOG_WARNING, "subscr unauth %s class %s",
 		       sender->string, subs->dest.classname->string);
 		continue; /* the for loop */
 	    }
 	    if (wildcard_instance == subs->dest.inst) {
-		if (!access_check(sender->string, acl, INSTWILD)) {
+		if (!access_check(sender->string, who, acl, INSTWILD)) {
 		    syslog(LOG_WARNING,
 			   "subscr unauth %s class %s wild inst",
 			   sender->string, subs->dest.classname->string);
@@ -1439,19 +1238,24 @@ Code_t subscr_foreign_user(ZNotice_t *notice,
   Client *client;
   ZNotice_t snotice;
   struct sockaddr_in newwho;
-  char *cp, *tp0, *tp1;
+  char *cp, *tp0;
+#ifdef DEBUG
+  char *tp1;
+#endif
   char rlm_recipient[REALM_SZ + 1];
 
   tp0 = cp = notice->z_message;
 
   newwho.sin_addr.s_addr = inet_addr(cp);
-  if (newwho.sin_addr.s_addr == -1) {
+  if (newwho.sin_addr.s_addr == INADDR_NONE) {
     syslog(LOG_ERR, "malformed addr from %s", notice->z_sender);
     return(ZERR_NONE);
   }
 
   I_ADVANCE(0);
+#ifdef DEBUG
   tp1 = cp;
+#endif
 
   snotice = *notice;
 
