@@ -3,7 +3,7 @@
  *
  *      Created by:     David C. Jedlinsky
  *
- *      $Id$
+ *      $Id: 333bf3e87dd53e32d2ce4260ad71111ba86c907f $
  *
  *      Copyright (c) 1987 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -14,7 +14,7 @@
 
 #ifndef lint
 #ifndef SABER
-static const char rcsid_queue_c[] = "$Id$";
+static const char rcsid_queue_c[] = "$Id: 333bf3e87dd53e32d2ce4260ad71111ba86c907f $";
 #endif /* SABER */
 #endif /* lint */
 
@@ -23,6 +23,7 @@ typedef struct _Queue {
     int retries;
     ZNotice_t notice;
     void *packet;
+    int len;
     struct sockaddr_in reply;
     struct _Queue *next, **prev_p;
 } Queue;
@@ -33,12 +34,10 @@ static int retransmits_enabled = 0;
 static Queue *find_notice_in_queue(ZNotice_t *notice);
 static void queue_timeout(void *arg);
 
-extern void new_server(char *);
-
 int rexmit_times[] = { 2, 2, 4, 4, 8, -1 };
 
 #ifdef DEBUG
-Code_t dump_queue(void);
+static void dump_queue(void);
 #endif
 
 void
@@ -78,6 +77,7 @@ add_notice_to_queue(ZNotice_t *notice,
 	    return(ZERR_NONOTICE);
 	}
 	memcpy(entry->packet, packet, Z_MAXPKTLEN);
+	entry->len = len;
 	if (ZParseNotice(entry->packet, len, &entry->notice) != ZERR_NONE) {
 	    syslog(LOG_ERR, "ZParseNotice failed, but succeeded before");
 	    free(entry->packet);
@@ -146,7 +146,7 @@ retransmit_queue(struct sockaddr_in *sin)
 	DPR2("\tz_opcode: %s\n", entry->notice.z_opcode);
 	DPR2("\tz_sender: %s\n", entry->notice.z_sender);
 	DPR2("\tz_recip: %s\n", entry->notice.z_recipient);
-	ret = send_outgoing(&entry->notice);
+	ret = ZSendPacket(entry->packet, entry->len, 0);
 	if (ret != ZERR_NONE) {
 	    Zperr(ret);
 	    com_err("queue", ret, "sending raw notice");
@@ -172,11 +172,11 @@ disable_queue_retransmits(void)
 }
 
 #ifdef DEBUG
-static Code_t
+static void
 dump_queue(void)
 {
     Queue *entry;
-    void *mp;
+    char *mp;
     int ml;
 
     DPR("Dumping queue...\n");
@@ -253,7 +253,7 @@ queue_timeout(void *arg)
     DPR2("\tz_opcode: %s\n", entry->notice.z_opcode);
     DPR2("\tz_sender: %s\n", entry->notice.z_sender);
     DPR2("\tz_recip: %s\n", entry->notice.z_recipient);
-    ret = send_outgoing(&entry->notice);
+    ret = ZSendPacket(entry->packet, entry->len, 0);
     if (ret != ZERR_NONE) {
 	Zperr(ret);
 	com_err("queue", ret, "sending raw notice");
