@@ -3,7 +3,7 @@
  *
  *      Created by:     David C. Jedlinsky
  *
- *      $Id$
+ *      $Id: ec4696b7f5c26c8685a5da35b8826bffcced6fc9 $
  *
  *      Copyright (c) 1987,1991 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -13,7 +13,7 @@
 #include "zhm.h"
 #include <zephyr_version.h>
 
-static const char rcsid_hm_c[] = "$Id$";
+static const char rcsid_hm_c[] = "$Id: ec4696b7f5c26c8685a5da35b8826bffcced6fc9 $";
 
 #ifdef HAVE_HESIOD
 int use_hesiod = 0;
@@ -28,7 +28,6 @@ int use_hesiod = 0;
 int hmdebug, rebootflag, noflushflag, errflg, dieflag, inetd, oldpid, nofork;
 int no_server = 1, nservchang, nserv, nclt;
 int booting = 1, timeout_type, deactivated = 1;
-int bootflag = 1;
 int started = 0;
 long starttime;
 u_short cli_port;
@@ -47,16 +46,12 @@ static RETSIGTYPE deactivate(int);
 static RETSIGTYPE terminate(int);
 static void choose_server(void);
 static void init_hm(void);
+#ifndef DEBUG
 static void detach(void);
+#endif
 static void send_stats(ZNotice_t *, struct sockaddr_in *);
 static char *strsave(const char *);
 
-extern void send_flush_notice(char *);
-extern void server_manager(ZNotice_t *);
-extern void send_boot_notice(char *);
-extern void find_next_server(char *);
-extern int optind;
-
 static RETSIGTYPE
 deactivate(int ignored)
 {
@@ -112,9 +107,9 @@ main(int argc,
 	  case 'f':
 	    noflushflag = 1;
 	    break;
-          case 'N':
-            bootflag = 0;
-            break;
+	  case 'N':
+	    booting = 0;
+	    break;
 	  case '?':
 	  default:
 	    errflg++;
@@ -258,10 +253,10 @@ main(int argc,
 static void
 choose_server(void)
 {
+#ifdef HAVE_HESIOD
     int i = 0;
     char **clust_info, **cpp;
 
-#ifdef HAVE_HESIOD
     if (use_hesiod) {
 
 	/* Free up any previously used resources */
@@ -368,7 +363,9 @@ init_hm(void)
 {
      struct servent *sp;
      Code_t ret;
+#ifndef DEBUG
      FILE *fp;
+#endif
 #ifdef _POSIX_VERSION
      struct sigaction sa;
 #endif
@@ -446,7 +443,7 @@ init_hm(void)
 	  memcpy(&serv_sin.sin_addr, hp->h_addr, 4);
      }
 
-     if (bootflag)
+     if (booting)
           send_boot_notice(HM_BOOT);
      else
           send_boot_notice(HM_ATTACH);
@@ -465,6 +462,7 @@ init_hm(void)
 #endif
 }
 
+#ifndef DEBUG
 static void
 detach(void)
 {
@@ -504,6 +502,7 @@ detach(void)
      setsid();
 #endif
 }
+#endif
 
 static char version[BUFSIZ];
 
