@@ -3,7 +3,7 @@
  *
  *      Created by:     David C. Jedlinsky
  *
- *      $Id$
+ *      $Id: 4c584287e79ae70ae9f1985e3496581896d1c8e0 $
  *
  *      Copyright (c) 1987 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -14,7 +14,7 @@
 
 #ifndef lint
 #ifndef SABER
-static const char rcsid_hm_server_c[] = "$Id$";
+static const char rcsid_hm_server_c[] = "$Id: 4c584287e79ae70ae9f1985e3496581896d1c8e0 $";
 #endif /* SABER */
 #endif /* lint */
 
@@ -26,18 +26,9 @@ static int serv_rexmit_times[] = { 5, 10, 20, 40 };
 static int serv_timeouts = 0;
 
 int serv_loop = 0;
-extern u_short cli_port;
-extern struct sockaddr_in serv_sin, from;
-extern int timeout_type, hmdebug, nservchang, booting, nserv, no_server;
-extern int deactivated, rebootflag;
-extern int numserv;
-extern char **serv_list;
-extern char cur_serv[], prim_serv[];
-extern void die_gracefully(void);
 
 void hm_control(ZNotice_t *);
 void send_back(ZNotice_t *);
-void new_server(char *);
 
 /* Argument is whether we are actually booting, or just attaching
  * after a server switch */
@@ -47,6 +38,7 @@ send_boot_notice(char *op)
      ZNotice_t notice;
      Code_t ret;
 
+     memset(&notice, 0, sizeof(ZNotice_t));
      /* Set up server notice */
      notice.z_kind = HMCTL;
      notice.z_port = cli_port;
@@ -78,6 +70,7 @@ send_flush_notice(char *op)
      ZNotice_t notice;
      Code_t ret;
 
+     memset(&notice, 0, sizeof(ZNotice_t));
      /* Set up server notice */
      notice.z_kind = HMCTL;
      notice.z_port = cli_port;
@@ -104,7 +97,7 @@ send_flush_notice(char *op)
 void
 find_next_server(char *sugg_serv)
 {
-     struct hostent *hp;
+     struct hostent *hp = 0;
      int done = 0;
      char **parse = serv_list;
      char *new_serv;
@@ -292,7 +285,7 @@ new_server(char *sugg_serv)
     syslog (LOG_INFO, "Server went down, finding new server.");
     send_flush_notice(HM_DETACH);
     find_next_server(sugg_serv);
-    if (booting) {
+    if (booting || deactivated) {
 	send_boot_notice(HM_BOOT);
 	deactivated = 0;
     } else {
