@@ -7,7 +7,7 @@
  *
  *      Created by:     Marc Horowitz <marc@athena.mit.edu>
  *
- *      $Id$
+ *      $Id: 7a77b91406a8cfe86e0aa0c0738c099fa154079b $
  *
  *      Copyright (c) 1989 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -21,8 +21,10 @@
 
 #include "new_string.h"
 
-extern Display *dpy;
+extern Display *x_dpy;
 
+extern char *X_driver(string);
+extern int X_driver_init(char *, char, int *, char **);
 extern char *get_string_resource(string, string);
 extern int get_bool_resource(string, string, int);
 extern unsigned long x_string_to_color(char *, unsigned long);
