@@ -4,7 +4,7 @@
  *
  *      Created by:     Marc Horowitz <marc@athena.mit.edu>
  *
- *      $Id$
+ *      $Id: 8d5bdd0fe84d687e6966f3b609e6eae8a1114ad4 $
  *
  *      Copyright (c) 1989 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -14,7 +14,7 @@
 #include <sysdep.h>
 
 #if (!defined(lint) && !defined(SABER))
-static const char rcsid_file_c[] = "$Id$";
+static const char rcsid_file_c[] = "$Id: 8d5bdd0fe84d687e6966f3b609e6eae8a1114ad4 $";
 #endif
 
 #include <zephyr/mit-copyright.h>
@@ -23,6 +23,7 @@ static const char rcsid_file_c[] = "$Id$";
 #include "new_memory.h"
 #include "new_string.h"
 #include "error.h"
+#include "file.h"
 
 /*
  *    char *get_home_directory()
@@ -38,7 +39,8 @@ static const char rcsid_file_c[] = "$Id$";
  *               if necessary.
  */
 
-char *get_home_directory(void)
+static char
+*get_home_directory(void)
 {
     char *result;
     struct passwd *passwd_entry;
