@@ -4,7 +4,7 @@
  *
  *      Created by:     Marc Horowitz <marc@athena.mit.edu>
  *
- *      $Id$
+ *      $Id: 65ac5c3c129b382ca86a8c3992dbb0c032b09adf $
  *
  *      Copyright (c) 1989 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -14,7 +14,7 @@
 #include <sysdep.h>
 
 #if (!defined(lint) && !defined(SABER))
-static const char rcsid_formatter_c[] = "$Id$";
+static const char rcsid_formatter_c[] = "$Id: 65ac5c3c129b382ca86a8c3992dbb0c032b09adf $";
 #endif
 
 #include <zephyr/mit-copyright.h>
@@ -235,7 +235,8 @@ verbatim(string str, int bracketsonly)
    char *temp,*temp2;
    int bracketnum,len;
 
-   if (strlen(str) == pure_text_length(str,0)) {
+   len = strlen(str);
+   if (len == pure_text_length(str,0)) {
       /* No environments, so consider the fast-and-easy methods */
 
       if (not_contains(str,allbracket_set)) {
