@@ -4,7 +4,7 @@
  *
  *      Created by:     Marc Horowitz <marc@athena.mit.edu>
  *
- *      $Id$
+ *      $Id: f59cdb83706e559cfdd1693aa79d0b6e88df9f59 $
  *
  *      Copyright (c) 1989 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -14,7 +14,7 @@
 #include <sysdep.h>
 
 #if (!defined(lint) && !defined(SABER))
-static const char rcsid_lexer_c[] = "$Id$";
+static const char rcsid_lexer_c[] = "$Id: f59cdb83706e559cfdd1693aa79d0b6e88df9f59 $";
 #endif
 
 #include <zephyr/mit-copyright.h>
@@ -186,7 +186,7 @@ lex_open(FILE *file)
      * Initialize keyword_dict from keywords if needed:
      */
     if (!keyword_dict) {
-	int i;
+	unsigned int i;
 
 	keyword_dict = int_dictionary_Create(101);
 
