@@ -4,7 +4,7 @@
  *
  *      Created by:     Marc Horowitz <marc@athena.mit.edu>
  *
- *      $Id$
+ *      $Id: a7b2210b538e41707305d881a2b1a3f7f0c4988a $
  *
  *      Copyright (c) 1989 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -14,7 +14,7 @@
 #include <sysdep.h>
 
 #if (!defined(lint) && !defined(SABER))
-static const char rcsid_mux_c[] = "$Id$";
+static const char rcsid_mux_c[] = "$Id: a7b2210b538e41707305d881a2b1a3f7f0c4988a $";
 #endif
 
 /****************************************************************************/
@@ -210,8 +210,9 @@ mux_loop(void)
 #ifdef DEBUG
 	      if (zwgc_debug)
 		fprintf(stderr,
-			"mux_loop...activity on fd %d, calling %x(%x)\n",
-			i,input_handler[i],input_handler_arg[i]);
+			"mux_loop...activity on fd %d, calling %lx(%lx)\n",
+			i, (unsigned long)input_handler[i],
+			 (unsigned long)input_handler_arg[i]);
 #endif
 	      input_handler[i](input_handler_arg[i]);
 	  }
