@@ -4,7 +4,7 @@
  *
  *      Created by:     Marc Horowitz <marc@athena.mit.edu>
  *
- *      $Id$
+ *      $Id: 31235fb8b148a69ce77168fe3c19498b19ed11a2 $
  *
  *      Copyright (c) 1989 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -17,10 +17,10 @@
 #ifndef memory_MODULE
 #define memory_MODULE
 
-extern char *memory__malloc();           /* PRIVATE */
-extern char *memory__realloc();          /* PRIVATE */
-extern char *memory__calloc();           /* PRIVATE */
-extern void memory__free();              /* PRIVATE */
+extern void *memory__malloc(unsigned);           /* PRIVATE */
+extern void *memory__realloc(void *, unsigned);  /* PRIVATE */
+extern void *memory__calloc(unsigned, unsigned); /* PRIVATE */
+extern void memory__free(void *);                /* PRIVATE */
 
 #ifdef DEBUG_MEMORY
 
