@@ -4,7 +4,7 @@
  *
  *      Created by:     Marc Horowitz <marc@athena.mit.edu>
  *
- *      $Id$
+ *      $Id: 4d952445cbda37f2acba35339ffebd5378df7cff $
  *
  *      Copyright (c) 1989 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -12,9 +12,10 @@
  */
 
 #include <sysdep.h>
+#include "new_string.h"
 
 #if (!defined(lint) && !defined(SABER))
-static const char rcsid_new_string_c[] = "$Id$";
+static const char rcsid_new_string_c[] = "$Id: 4d952445cbda37f2acba35339ffebd5378df7cff $";
 #endif
 
 /*
@@ -43,9 +44,6 @@ static const char rcsid_new_string_c[] = "$Id$";
 
 #include "new_memory.h"
 
-#define string_Length(s) strlen(s)
-typedef char *string;
-
 /*
  *    string string_CreateFromData(char *data, int length):
  *        Requires: data[0], data[1], ..., data[length-1] != 0
