@@ -4,7 +4,7 @@
  *
  *      Created by:     Marc Horowitz <marc@athena.mit.edu>
  *
- *      $Id$
+ *      $Id: 45ac997da054e5dea959e0d24046efe77225b627 $
  *
  *      Copyright (c) 1989 by the Massachusetts Institute of Technology.
  *      For copying and distribution information, see the file
@@ -14,7 +14,7 @@
 #include <sysdep.h>
 
 #if (!defined(lint) && !defined(SABER))
-static const char rcsid_xselect_c[] = "$Id$";
+static const char rcsid_xselect_c[] = "$Id: 45ac997da054e5dea959e0d24046efe77225b627 $";
 #endif
 
 #include <zephyr/mit-copyright.h>
@@ -96,17 +96,18 @@ xselSetProperties(Display *dpy,
    } else if (target==ZA_MULTIPLE) {
       Atom atype;
       int aformat;
+      unsigned char *alistp; /* Avoid strict aliasing violation, we hope */
       Atom *alist;
       unsigned long alistsize,i;
 
       XGetWindowProperty(dpy,w,property,0L,0L,False,ZA_ATOM_PAIR,&atype,
-			 &aformat,&i,&alistsize,(unsigned char **) &alist);
+			 &aformat,&i,&alistsize,&alistp);
 
       if (alistsize)
 	XGetWindowProperty(dpy,w,property,0L,alistsize/sizeof(Atom),False,
-			   ZA_ATOM_PAIR,&atype,&aformat,&alistsize,&i,
-			   (unsigned char **) &alist);
+			   ZA_ATOM_PAIR,&atype,&aformat,&alistsize,&i, &alistp);
 
+      alist = (Atom *)alistp;
       alistsize/=(sizeof(Atom)/4);
       for (i=0;i<alistsize;i+=2)
 	xselSetProperties(dpy,w,alist[i+1],alist[i],selreq);
@@ -126,7 +127,7 @@ xselSetProperties(Display *dpy,
 	    spec (or if this program is buggy), but it could happen */
 #ifdef DEBUG
 	 fprintf(stderr,
-		 "SelectionRequest event received for unowned selection: requestor wid=0x%x", w);
+		 "SelectionRequest event received for unowned selection: requestor wid=0x%lx", (unsigned long)w);
 #endif
 	 ChangeProp(XA_STRING,8,"",0);
       }
@@ -141,7 +142,7 @@ xselSetProperties(Display *dpy,
 void
 xicccmInitAtoms(Display *dpy)
 {
-   int i;
+   unsigned int i;
 
    for (i=0;i<NumZAtoms;i++)
      *(ZAtom[i].patom)=XInternAtom(dpy,ZAtom[i].name,False);
@@ -153,15 +154,15 @@ xicccmInitAtoms(Display *dpy)
 int
 xselGetOwnership(Display *dpy,
 		 Window w,
-		 Time time)
+		 Time when)
 {
    int temp;
 
-   XSetSelectionOwner(dpy,XA_PRIMARY,w,time);
+   XSetSelectionOwner(dpy,XA_PRIMARY,w,when);
    temp=(w == XGetSelectionOwner(dpy,XA_PRIMARY));
 
    if (temp)
-     ownership_start = time;
+     ownership_start = when;
 
    return(temp);
 }
@@ -194,9 +195,9 @@ xselProcessSelection(Display *dpy,
 }
 
 void
-xselOwnershipLost(Time time)
+xselOwnershipLost(Time when)
 {
-   ownership_end = time;
+   ownership_end = when;
 }
 
 /*ARGSUSED*/
